# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------
import base64
import json
import random
import unittest

from azure.cli.core import azclierror
from azure.cli.core.aaz import exceptions as aazerror
from azure.cli.core.aaz._command_ctx import AAZCommandCtx
from azure.cli.core.aaz import AAZArgumentsSchema
from azure.cli.core.mock import DummyCli


class TestAAZArgBaseFmt(unittest.TestCase):

    @staticmethod
    def format_arg(schema, data):
        ctx = AAZCommandCtx(
            cli_ctx=DummyCli(), schema=schema,
            command_args=data
        )
        ctx.format_args()
        return ctx.args

    def test_str_fmt(self):
        from azure.cli.core.aaz import AAZStrArg, AAZStrArgFormat
        schema = AAZArgumentsSchema()
        schema.str1 = AAZStrArg(
            fmt=AAZStrArgFormat(
                pattern="[a-z]+",
                max_length=8,
                min_length=2,
            ),
            nullable=True
        )

        with self.assertRaises(azclierror.InvalidArgumentValueError):
            self.format_arg(schema, {"str1": ""})

        with self.assertRaises(azclierror.InvalidArgumentValueError):
            self.format_arg(schema, {"str1": "1234"})

        with self.assertRaises(azclierror.InvalidArgumentValueError):
            self.format_arg(schema, {"str1": "abcdefghi"})

        with self.assertRaises(azclierror.InvalidArgumentValueError):
            self.format_arg(schema, {"str1": "aBCD"})

        args = self.format_arg(schema, {"str1": "abcdefgh"})
        self.assertEqual(args.str1, "abcdefgh")

        args = self.format_arg(schema, {"str1": "abcd"})
        self.assertEqual(args.str1, "abcd")

        args = self.format_arg(schema, {"str1": None})
        self.assertEqual(args.str1, None)

    def test_duration_fmt(self):
        from azure.cli.core.aaz import AAZDurationArg, AAZDurationFormat
        schema = AAZArgumentsSchema()
        schema.duration = AAZDurationArg(
            nullable=True
        )

        args = self.format_arg(schema, {"duration": "1s"})
        self.assertEqual(args.duration, "PT1S")

        args = self.format_arg(schema, {"duration": "1m"})
        self.assertEqual(args.duration, "PT1M")

        args = self.format_arg(schema, {"duration": "1h"})
        self.assertEqual(args.duration, "PT1H")

        args = self.format_arg(schema, {"duration": "1d"})
        self.assertEqual(args.duration, "P1D")

        args = self.format_arg(schema, {"duration": "0h1s"})
        self.assertEqual(args.duration, "PT1S")

        args = self.format_arg(schema, {"duration": "1dt1h1m1s"})
        self.assertEqual(args.duration, "P1DT1H1M1S")

        args = self.format_arg(schema, {"duration": "pt1h"})
        self.assertEqual(args.duration, "PT1H")

        args = self.format_arg(schema, {"duration": "p1yt1h"})
        self.assertEqual(args.duration, "P1YT1H")

        args = self.format_arg(schema, {"duration": "p1y"})
        self.assertEqual(args.duration, "P1Y")

        args = self.format_arg(schema, {"duration": "p1m"})
        self.assertEqual(args.duration, "P1M")

        args = self.format_arg(schema, {"duration": "P1Y1M100DT1h1s"})
        self.assertEqual(args.duration, "P1Y1M100DT1H1S")

        args = self.format_arg(schema, {"duration": None})
        self.assertEqual(args.duration, None)

        with self.assertRaises(azclierror.InvalidArgumentValueError):
            self.format_arg(schema, {"duration": "1m1d"})

        with self.assertRaises(azclierror.InvalidArgumentValueError):
            self.format_arg(schema, {"duration": "1x"})

        with self.assertRaises(azclierror.InvalidArgumentValueError):
            self.format_arg(schema, {"duration": "-1m"})

        with self.assertRaises(azclierror.InvalidArgumentValueError):
            self.format_arg(schema, {"duration": "m1s"})

    def test_date_fmt(self):
        from azure.cli.core.aaz import AAZDateArg, AAZDateFormat
        schema = AAZArgumentsSchema()
        schema.date = AAZDateArg(
            nullable=True
        )

        args = self.format_arg(schema, {"date": "2010-01-02 01:11:11 +02:00"})
        self.assertEqual(args.date, "2010-01-02")

        args = self.format_arg(schema, {"date": "2010/01/02"})
        self.assertEqual(args.date, "2010-01-02")

        args = self.format_arg(schema, {"date": "October 1 1989"})
        self.assertEqual(args.date, "1989-10-01")

        args = self.format_arg(schema, {"date": "2010/01/02 01:11:11"})
        self.assertEqual(args.date, "2010-01-02")

        args = self.format_arg(schema, {"date": None})
        self.assertEqual(args.date, None)

        with self.assertRaises(azclierror.InvalidArgumentValueError):
            args = self.format_arg(schema, {"date": "aaaa"})

    def test_time_fmt(self):
        from azure.cli.core.aaz import AAZTimeArg, AAZTimeFormat
        schema = AAZArgumentsSchema()
        schema.time = AAZTimeArg(
            nullable=True
        )

        args = self.format_arg(schema, {"time": "2010-01-02 01:11:11.12345678 +01:00"})
        self.assertEqual(args.time, "01:11:11.123456")

        args = self.format_arg(schema, {"time": "01:11:11.0001"})
        self.assertEqual(args.time, "01:11:11.0001")

        args = self.format_arg(schema, {"time": "21:11:11.1"})
        self.assertEqual(args.time, "21:11:11.100")

        args = self.format_arg(schema, {"time": "21:01:51.00"})
        self.assertEqual(args.time, "21:01:51")

        args = self.format_arg(schema, {"time": None})
        self.assertEqual(args.time, None)

        with self.assertRaises(azclierror.InvalidArgumentValueError):
            self.format_arg(schema, {"time": "21:01:151.00"})

    def test_datetime_fmt(self):
        from azure.cli.core.aaz import AAZDateTimeArg, AAZDateTimeFormat
        schema = AAZArgumentsSchema()
        schema.datetime = AAZDateTimeArg(
            nullable=True
        )
        schema.rfc = AAZDateTimeArg(
            fmt=AAZDateTimeFormat(protocol='rfc'),
            nullable=True
        )

        args = self.format_arg(schema, {
            "datetime": "2010-01-02 01:11:11.12345678 -06:30",
            "rfc": "2010-01-02 01:11:11.12345678 -06:30",
        })
        self.assertEqual(args.datetime, "2010-01-02T07:41:11.123456Z")
        self.assertEqual(args.rfc, "Sat, 02 Jan 2010 07:41:11 GMT")

        args = self.format_arg(schema, {
            "datetime": "Sat, 02 Jan 2010 07:41:11 GMT",
            "rfc": "2010-01-02T07:41:11.123456Z",
        })
        self.assertEqual(args.datetime, "2010-01-02T07:41:11.000Z")
        self.assertEqual(args.rfc, "Sat, 02 Jan 2010 07:41:11 GMT")

        args = self.format_arg(schema, {
            "datetime": "2010/01/02 01:11:11.12345678 +06:30",
            "rfc": "2010/01/02 01:11:11.12345678 +06:30",
        })
        self.assertEqual(args.datetime, "2010-01-01T18:41:11.123456Z")
        self.assertEqual(args.rfc, "Fri, 01 Jan 2010 18:41:11 GMT")

        args = self.format_arg(schema, {
            "datetime": "2010-01-02T00:00:00Z",
            "rfc": "2010-01-02T00:00:00Z",
        })
        self.assertEqual(args.datetime, "2010-01-02T00:00:00.000Z")
        self.assertEqual(args.rfc, "Sat, 02 Jan 2010 00:00:00 GMT")

        args = self.format_arg(schema, {
            "datetime": "2010-01-02 10:00:00 +00:00",
            "rfc": "2010-01-02 10:00:00 +00:00",
        })
        self.assertEqual(args.datetime, "2010-01-02T10:00:00.000Z")
        self.assertEqual(args.rfc, "Sat, 02 Jan 2010 10:00:00 GMT")

        args = self.format_arg(schema, {
            "datetime": None,
            "rfc": None,
        })
        self.assertEqual(args.datetime, None)
        self.assertEqual(args.rfc, None)

        with self.assertRaises(azclierror.InvalidArgumentValueError):
            args = self.format_arg(schema, {
                "datetime": "-2010-01-a2 10:00:00 +00:00",
            })

    def test_uuid_fmt(self):
        from azure.cli.core.aaz import AAZUuidArg, AAZUuidFormat
        schema = AAZArgumentsSchema()
        schema.uuid = AAZUuidArg(
            nullable=True
        )

        with self.assertRaises(azclierror.InvalidArgumentValueError):
            self.format_arg(schema, {"uuid": ""})

        with self.assertRaises(azclierror.InvalidArgumentValueError):
            self.format_arg(schema, {"uuid": "a8577a7a-4f31-40ab-bb00-3557df24a7e"})

        with self.assertRaises(azclierror.InvalidArgumentValueError):
            self.format_arg(schema, {"uuid": "a8577a7a-4f31-40ab-bb003557df24a7ea"})

        with self.assertRaises(azclierror.InvalidArgumentValueError):
            self.format_arg(schema, {"uuid": "a8577Z7Z-4f31-40ab-bb00-3557df24a7ea"})

        with self.assertRaises(azclierror.InvalidArgumentValueError):
            self.format_arg(schema, {"uuid": "{a8577a7a-4f31-40ab-bb00-3557df24a7ea"})

        with self.assertRaises(azclierror.InvalidArgumentValueError):
            self.format_arg(schema, {"uuid": "a8577a7a-4f31-40ab-bb00-3557df24a7ea}"})

        with self.assertRaises(azclierror.InvalidArgumentValueError):
            self.format_arg(schema, {"uuid": "(a8577a7a-4f31-40ab-bb00-3557df24a7ea"})

        with self.assertRaises(azclierror.InvalidArgumentValueError):
            self.format_arg(schema, {"uuid": "a8577a7a-4f31-40ab-bb00-3557df24a7ea)"})

        args = self.format_arg(schema, {"uuid": "a8577a7a-4f31-40ab-bb00-3557df24a7ea"})
        self.assertEqual(args.uuid, "a8577a7a-4f31-40ab-bb00-3557df24a7ea")

        args = self.format_arg(schema, {"uuid": "FF0B2DF5-97EB-4E45-B028-8B0BEC803ABE"})
        self.assertEqual(args.uuid, "FF0B2DF5-97EB-4E45-B028-8B0BEC803ABE")

        args = self.format_arg(schema, {"uuid": "a8577A7a-4f31-40Ab-bb00-3557Df24a7Ea"})
        self.assertEqual(args.uuid, "a8577A7a-4f31-40Ab-bb00-3557Df24a7Ea")

        args = self.format_arg(schema, {"uuid": "{a8577a7a-4f31-40ab-bb00-3557df24a7ea}"})
        self.assertEqual(args.uuid, "a8577a7a-4f31-40ab-bb00-3557df24a7ea")

        args = self.format_arg(schema, {"uuid": "(a8577a7a-4f31-40ab-bb00-3557df24a7ea)"})
        self.assertEqual(args.uuid, "a8577a7a-4f31-40ab-bb00-3557df24a7ea")

        args = self.format_arg(schema, {"uuid": "a8577a7a4f3140abBb003557df24a7eA"})
        self.assertEqual(args.uuid, "a8577a7a-4f31-40ab-Bb00-3557df24a7eA")

        args = self.format_arg(schema, {"uuid": "{a8577a7a4f3140abbb003557df24a7ea}"})
        self.assertEqual(args.uuid, "a8577a7a-4f31-40ab-bb00-3557df24a7ea")

        args = self.format_arg(schema, {"uuid": "(a8577a7a4f3140abbb003557df24a7ea)"})
        self.assertEqual(args.uuid, "a8577a7a-4f31-40ab-bb00-3557df24a7ea")

        schema = AAZArgumentsSchema()
        schema.uuid = AAZUuidArg(
            fmt=AAZUuidFormat(case='upper'),
            nullable=True
        )

        args = self.format_arg(schema, {"uuid": "(a8577a7A4f3140Abbb003557df24a7ea)"})
        self.assertEqual(args.uuid, "A8577A7A-4F31-40AB-BB00-3557DF24A7EA")

        schema = AAZArgumentsSchema()
        schema.uuid = AAZUuidArg(
            fmt=AAZUuidFormat(case='lower'),
            nullable=True
        )

        args = self.format_arg(schema, {"uuid": "(A8577A7A4F3140Abbb003557DF24A7EA)"})
        self.assertEqual(args.uuid, "a8577a7a-4f31-40ab-bb00-3557df24a7ea")

        schema = AAZArgumentsSchema()
        schema.uuid = AAZUuidArg(
            fmt=AAZUuidFormat(hyphens_filled=False),
            nullable=True
        )

        args = self.format_arg(schema, {"uuid": "(A8577A7A4F3140Abbb003557DF24A7EA)"})
        self.assertEqual(args.uuid, "A8577A7A4F3140Abbb003557DF24A7EA")

        schema = AAZArgumentsSchema()
        schema.uuid = AAZUuidArg(
            fmt=AAZUuidFormat(braces_removed=False),
            nullable=True
        )

        args = self.format_arg(schema, {"uuid": "(a8577a7a4f3140abbb003557df24a7ea)"})
        self.assertEqual(args.uuid, "(a8577a7a-4f31-40ab-bb00-3557df24a7ea)")

        schema = AAZArgumentsSchema()
        schema.uuid = AAZUuidArg(
            fmt=AAZUuidFormat(braces_removed=False, hyphens_filled=False),
            nullable=True
        )

        args = self.format_arg(schema, {"uuid": "(A8577A7A4F3140Abbb003557DF24A7EA)"})
        self.assertEqual(args.uuid, "(A8577A7A4F3140Abbb003557DF24A7EA)")

        args = self.format_arg(schema, {"uuid": None})
        self.assertEqual(args.uuid, None)

    def test_int_fmt(self):
        from azure.cli.core.aaz import AAZIntArg, AAZIntArgFormat

        schema = AAZArgumentsSchema()
        schema.int1 = AAZIntArg(
            fmt=AAZIntArgFormat(
                multiple_of=10,
                maximum=30,
                minimum=20,
            ),
            nullable=True
        )

        with self.assertRaises(azclierror.InvalidArgumentValueError):
            self.format_arg(schema, {"int1": 10})

        with self.assertRaises(azclierror.InvalidArgumentValueError):
            self.format_arg(schema, {"int1": 25})

        with self.assertRaises(azclierror.InvalidArgumentValueError):
            self.format_arg(schema, {"int1": 40})

        args = self.format_arg(schema, {"int1": 20})
        self.assertEqual(args.int1, 20)

        args = self.format_arg(schema, {"int1": 30})
        self.assertEqual(args.int1, 30)

        args = self.format_arg(schema, {"int1": None})
        self.assertEqual(args.int1, None)

    def test_float_fmt(self):
        from azure.cli.core.aaz import AAZFloatArg, AAZFloatArgFormat

        schema = AAZArgumentsSchema()
        schema.flt1 = AAZFloatArg(
            fmt=AAZFloatArgFormat(
                multiple_of=1.1,
                maximum=33,
                minimum=22,
                exclusive_maximum=True,
                exclusive_minimum=True,
            )
        )

        with self.assertRaises(azclierror.InvalidArgumentValueError):
            self.format_arg(schema, {"flt1": 1.1})

        with self.assertRaises(azclierror.InvalidArgumentValueError):
            self.format_arg(schema, {"flt1": 22})

        with self.assertRaises(azclierror.InvalidArgumentValueError):
            self.format_arg(schema, {"flt1": 33})

        with self.assertRaises(azclierror.InvalidArgumentValueError):
            self.format_arg(schema, {"flt1": 23})

        with self.assertRaises(azclierror.InvalidArgumentValueError):
            self.format_arg(schema, {"flt1": 23.099})

        with self.assertRaises(azclierror.InvalidArgumentValueError):
            self.format_arg(schema, {"flt1": 31.901})

        args = self.format_arg(schema, {"flt1": 23.1})
        self.assertEqual(args.flt1, 23.1)

        args = self.format_arg(schema, {"flt1": 31.9})
        self.assertEqual(args.flt1, 31.9)

        with self.assertRaises(azclierror.InvalidArgumentValueError):
            self.format_arg(schema, {"flt1": 22.0000000001})

        with self.assertRaises(azclierror.InvalidArgumentValueError):
            self.format_arg(schema, {"flt1": 32.9999999999})

        schema = AAZArgumentsSchema()
        schema.flt1 = AAZFloatArg(
            fmt=AAZFloatArgFormat(
                multiple_of=1.1,
                maximum=33,
                minimum=22,
                exclusive_maximum=False,
                exclusive_minimum=False,
            ),
            nullable=True
        )
        args = self.format_arg(schema, {"flt1": 22.0000000001})
        self.assertEqual(args.flt1, 22)

        args = self.format_arg(schema, {"flt1": 32.9999999999})
        self.assertEqual(args.flt1, 33)

        args = self.format_arg(schema, {"flt1": None})
        self.assertEqual(args.flt1, None)

    def test_bool_fmt(self):
        from azure.cli.core.aaz import AAZBoolArg, AAZBoolArgFormat

        schema = AAZArgumentsSchema()
        schema.bool = AAZBoolArg(
            fmt=AAZBoolArgFormat(reverse=True),
            nullable=True,
        )

        args = self.format_arg(schema, {"bool": True})
        self.assertEqual(args.bool, False)

        args = self.format_arg(schema, {"bool": False})
        self.assertEqual(args.bool, True)

        args = self.format_arg(schema, {"bool": None})
        self.assertEqual(args.bool, None)

    def test_object_fmt(self):
        from azure.cli.core.aaz import AAZObjectArgFormat, AAZStrArgFormat, AAZBoolArgFormat, AAZIntArgFormat, \
            AAZDictArgFormat, AAZListArgFormat, AAZFloatArgFormat
        from azure.cli.core.aaz import AAZObjectArg, AAZStrArg, AAZBoolArg, AAZIntArg, AAZDictArg, AAZListArg, \
            AAZFloatArg

        schema = AAZArgumentsSchema()
        schema.properties = AAZObjectArg(fmt=AAZObjectArgFormat(
            max_properties=3,
            min_properties=2,
        ))
        schema.properties.name = AAZStrArg(fmt=AAZStrArgFormat(pattern='[a-z]+'))
        schema.properties.enabled = AAZBoolArg(fmt=AAZBoolArgFormat(reverse=True))
        schema.properties.count = AAZIntArg(fmt=AAZIntArgFormat(minimum=0))
        schema.properties.vnet = AAZObjectArg()
        schema.properties.vnet.name = AAZStrArg()
        schema.properties.vnet.name2 = AAZStrArg()

        args = self.format_arg(schema, {
            "properties": {
                "name": "abcd",
                "enabled": True,
                "count": 100
            }
        })
        self.assertEqual(args.properties.to_serialized_data(), {
            "name": "abcd",
            "enabled": False,
            "count": 100
        })

        with self.assertRaises(azclierror.InvalidArgumentValueError):
            self.format_arg(schema, {
                "properties": {
                    "name": "abcd",
                }
            })

        with self.assertRaises(azclierror.InvalidArgumentValueError):
            self.format_arg(schema, {
                "properties": {
                    "name": "abcd",
                    "enabled": False,
                    "count": 100,
                    "vnet": {
                        "name": "test",
                    }
                }
            })

        schema = AAZArgumentsSchema()
        schema.properties = AAZObjectArg(nullable=True)
        schema.properties.name = AAZStrArg(fmt=AAZStrArgFormat(pattern='[a-z]+'))
        schema.properties.enabled = AAZBoolArg(fmt=AAZBoolArgFormat(reverse=True))
        schema.properties.count = AAZIntArg(fmt=AAZIntArgFormat(minimum=0))
        schema.properties.vnet = AAZObjectArg(nullable=True)
        schema.properties.vnet.name = AAZStrArg(fmt=AAZStrArgFormat(pattern='[a-z]+'))

        schema.properties.tags = AAZDictArg(fmt=AAZDictArgFormat())
        schema.properties.tags.Element = AAZStrArg(fmt=AAZStrArgFormat(pattern='[a-z]+'))
        schema.properties.actions = AAZListArg(fmt=AAZListArgFormat())
        schema.properties.actions.Element = AAZStrArg(fmt=AAZStrArgFormat(pattern='[a-z]+'))
        schema.properties.sub = AAZObjectArg(fmt=AAZObjectArgFormat())
        schema.properties.name2 = AAZStrArg(fmt=AAZStrArgFormat())
        schema.properties.int2 = AAZIntArg(fmt=AAZIntArgFormat())
        schema.properties.bool2 = AAZBoolArg(fmt=AAZBoolArgFormat())
        schema.properties.float2 = AAZFloatArg(fmt=AAZFloatArgFormat())

        args = self.format_arg(schema, {
            "properties": {}
        })
        self.assertEqual(args.properties.to_serialized_data(), {})

        with self.assertRaises(azclierror.InvalidArgumentValueError):
            self.format_arg(schema, {
                "properties": {
                    "name": "a1234",
                }
            })

        with self.assertRaises(azclierror.InvalidArgumentValueError):
            self.format_arg(schema, {
                "properties": {
                    "count": -10,
                }
            })

        with self.assertRaises(azclierror.InvalidArgumentValueError):
            self.format_arg(schema, {
                "properties": {
                    "vnet": {
                        "name": "test11",
                    }
                }
            })

        args = self.format_arg(schema, {"properties": None})
        self.assertEqual(args.properties, None)

        args = self.format_arg(schema, {
            "properties": {
                "vnet": None
            }
        })
        self.assertEqual(args, {
            "properties": {
                "vnet": None
            }
        })

    def test_dict_fmt(self):
        from azure.cli.core.aaz import AAZObjectArgFormat, AAZDictArgFormat, AAZStrArgFormat, AAZIntArgFormat
        from azure.cli.core.aaz import AAZObjectArg, AAZDictArg, AAZStrArg, AAZIntArg

        schema = AAZArgumentsSchema()
        schema.tags = AAZDictArg(fmt=AAZDictArgFormat(max_properties=3, min_properties=2), nullable=True)
        schema.tags.Element = AAZStrArg(fmt=AAZStrArgFormat(pattern='[a-z0-9]+'))
        schema.actions = AAZDictArg()
        schema.actions.Element = AAZObjectArg(nullable=True)
        schema.actions.Element.name = AAZStrArg(fmt=AAZStrArgFormat(pattern='[a-z]+'))
        schema.actions.Element.name2 = AAZStrArg(nullable=True)

        args = self.format_arg(schema, {
            "tags": {
                "flag1": "v1",
                "flag2": "v2",
            },
            "actions": {
                "a": {
                    "name": "abc"
                },
                "b": {
                    "name": "c"
                }
            }
        })
        self.assertEqual(args.to_serialized_data(), {
            "tags": {
                "flag1": "v1",
                "flag2": "v2",
            },
            "actions": {
                "a": {
                    "name": "abc"
                },
                "b": {
                    "name": "c"
                }
            }
        })

        with self.assertRaises(azclierror.InvalidArgumentValueError):
            self.format_arg(schema, {
                "tags": {
                    "flag1": "v1",
                },
            })

        with self.assertRaises(azclierror.InvalidArgumentValueError):
            self.format_arg(schema, {
                "tags": {
                    "flag1": "v1",
                    "flag2": "v2",
                    "flag3": "v3",
                    "flag4": "v4",
                },
            })

        with self.assertRaises(azclierror.InvalidArgumentValueError):
            self.format_arg(schema, {
                "actions": {
                    "a": {
                        "name": "abc11"
                    },
                },
            })

        args = self.format_arg(schema, {
            "tags": None,
            "actions": {
                "a": {
                    "name": "abc"
                },
                "b": {
                    "name": "c",
                    "name2": None,
                },
                "c": None
            }
        })
        self.assertEqual(args.to_serialized_data(), {
            "tags": None,
            "actions": {
                "a": {
                    "name": "abc"
                },
                "b": {
                    "name": "c",
                    "name2": None,
                },
                "c": None
            }
        })

    def test_freeform_dict_fmt(self):
        from azure.cli.core.aaz import AAZFreeFormDictArgFormat
        from azure.cli.core.aaz import AAZFreeFormDictArg

        schema = AAZArgumentsSchema()
        schema.tags = AAZFreeFormDictArg(fmt=AAZFreeFormDictArgFormat(max_properties=3, min_properties=2), nullable=True)

        args = self.format_arg(schema, {
            "tags": {
                "flag1": "v1",
                "flag2": 2,
            },
        })
        self.assertEqual(args.to_serialized_data(), {
            "tags": {
                "flag1": "v1",
                "flag2": 2,
            }
        })

        with self.assertRaises(azclierror.InvalidArgumentValueError):
            self.format_arg(schema, {
                "tags": {
                    "flag1": "v1",
                },
            })

        with self.assertRaises(azclierror.InvalidArgumentValueError):
            self.format_arg(schema, {
                "tags": {
                    "flag1": "v1",
                    "flag2": "v2",
                    "flag3": "v3",
                    "flag4": "v4",
                },
            })

        args = self.format_arg(schema, {
            "tags": None,
        })
        self.assertEqual(args.to_serialized_data(), {
            "tags": None
        })

    def test_list_fmt(self):
        from azure.cli.core.aaz import AAZObjectArgFormat, AAZListArgFormat, AAZStrArgFormat, AAZIntArgFormat
        from azure.cli.core.aaz import AAZObjectArg, AAZListArg, AAZStrArg, AAZIntArg

        schema = AAZArgumentsSchema()
        schema.tags = AAZListArg(fmt=AAZListArgFormat(unique=True, max_length=3, min_length=2))
        schema.tags.Element = AAZStrArg(fmt=AAZStrArgFormat(pattern='[a-z0-9]+'), nullable=True)
        schema.actions = AAZListArg(nullable=True)
        schema.actions.Element = AAZObjectArg()
        schema.actions.Element.name = AAZStrArg(fmt=AAZStrArgFormat(pattern='[a-z]+'))

        args = self.format_arg(schema, {
            "tags": ['v1', 'v2'],
            "actions": [
                {
                    "name": "abc"
                },
                {
                    "name": "c"
                }
            ]
        })
        self.assertEqual(args.to_serialized_data(), {
            "tags": ['v1', 'v2'],
            "actions": [
                {
                    "name": "abc"
                },
                {
                    "name": "c"
                }
            ]
        })

        with self.assertRaises(azclierror.InvalidArgumentValueError):
            self.format_arg(schema, {
                "tags": ['v1',],
            })

        with self.assertRaises(azclierror.InvalidArgumentValueError):
            self.format_arg(schema, {
                "tags": ['v1', 'v2', 'v3', 'v4'],
            })

        with self.assertRaises(azclierror.InvalidArgumentValueError):
            self.format_arg(schema, {
                "tags": ['v1', 'v2', 'v2'],
            })

        args = self.format_arg(schema, {
            "tags": ['v1', 'v2'],
        })
        self.assertEqual(args.to_serialized_data(), {
            "tags": ['v1', 'v2'],
        })

        args = self.format_arg(schema, {
            "tags": ['v1', 'v2', None],
            "actions": None,
        })
        self.assertEqual(args.to_serialized_data(), {
            "tags": ['v1', 'v2', None],
            "actions": None,
        })

    def test_aaz_file_arg_fmt(self):
        from azure.cli.core.aaz import AAZArgumentsSchema, AAZFileArg, AAZFileArgTextFormat, \
            AAZFileArgBase64EncodeFormat
        import os
        import random
        schema = AAZArgumentsSchema()
        v = schema()

        schema.text = AAZFileArg(fmt=AAZFileArgTextFormat())
        test_file = "test_aaz_file_arg_fmt.txt"
        content = "This is test"
        with open(test_file, 'w') as f:
            f.write(content)

        args = self.format_arg(schema, {
            "text": test_file,
        })

        self.assertEqual(args.to_serialized_data(), {
            "text": content
        })

        os.remove(test_file)
        with self.assertRaises(azclierror.InvalidArgumentValueError):
            self.format_arg(schema, {
                "text": test_file,
            })

        schema.data = AAZFileArg(fmt=AAZFileArgBase64EncodeFormat())
        data_file = "test_aaz_file_arg_fmt.data"
        data = bytes([random.randrange(0, 256) for _ in range(0, 128)])
        with open(data_file, 'wb') as f:
            f.write(data)

        args = self.format_arg(schema, {
            "data": data_file,
        })
        self.assertEqual(args.to_serialized_data(), {
            "data": base64.b64encode(data).decode("utf-8")
        })

        os.remove(data_file)
        with self.assertRaises(azclierror.InvalidArgumentValueError):
            self.format_arg(schema, {
                "data": data_file,
            })

    def test_aaz_pagination_token_fmt(self):
        from azure.cli.core.aaz import AAZArgumentsSchema, AAZPaginationTokenArg, AAZPaginationTokenArgFormat
        schema = AAZArgumentsSchema()
        schema.token = AAZPaginationTokenArg(fmt=AAZPaginationTokenArgFormat())

        # {next_link: None, offset: 0}
        data = "eyJuZXh0X2xpbmsiOiBudWxsLCAib2Zmc2V0IjogMH0="
        args = self.format_arg(schema, {"token": data})
        self.assertEqual(args.to_serialized_data(), {"token": base64.b64decode(data).decode("utf-8")})

        # incomplete base64 string
        data = "eyJuZXh0X2xpbmsiOiBudWxsLCAib2Zmc2V0IjogMH0"
        with self.assertRaises(azclierror.InvalidArgumentValueError) as e:
            self.format_arg(schema, {"token": data})
        self.assertEqual(str(e.exception), "InvalidArgumentValue: --next-token: Invalid Base64 string.")

        # utf-16 encoded
        data = "//57ACIAbgBlAHgAdABfAGwAaQBuAGsAIgA6ACAAbgB1AGwAbAAsACAAIgBvAGYAZgBzAGUAdAAiADoAIAAwAH0A"
        with self.assertRaises(azclierror.InvalidArgumentValueError) as e:
            self.format_arg(schema, {"token": data})
        self.assertEqual(str(e.exception), "InvalidArgumentValue: --next-token: Error decoding UTF-8.")

        # {next_link: None, offset: 0,}
        data = "eyJuZXh0X2xpbmsiOiBudWxsLCAib2Zmc2V0IjogMCx9"
        with self.assertRaises(azclierror.InvalidArgumentValueError) as e:
            self.format_arg(schema, {"token": data})
        self.assertEqual(str(e.exception), "InvalidArgumentValue: --next-token: Invalid JSON object.")

        # ["next_link", null, "offset", 0]
        data = "WyJuZXh0X2xpbmsiLCBudWxsLCAib2Zmc2V0IiwgMF0="
        with self.assertRaises(azclierror.InvalidArgumentValueError) as e:
            self.format_arg(schema, {"token": data})
        self.assertEqual(str(e.exception), "InvalidArgumentValue: --next-token: Decoded object is not a dictionary.")

        # {"curr_link": None, "offset": 0}
        data = "eyJjdXJyX2xpbmsiOiBudWxsLCAib2Zmc2V0IjogMH0="
        with self.assertRaises(azclierror.InvalidArgumentValueError) as e:
            self.format_arg(schema, {"token": data})
        self.assertEqual(str(e.exception), "InvalidArgumentValue: --next-token: `next_link` or `offset` doesn't exist.")

        def _encode(obj):
            return base64.b64encode(json.dumps(obj).encode("utf-8")).decode("utf-8")

        # next_link pointing to the trusted ARM endpoint is accepted
        valid_next_link = "https://management.azure.com/subscriptions/00000000-0000-0000-0000-000000000000/resources?api-version=2025-09-01"
        data = _encode({"next_link": valid_next_link, "offset": 0})
        args = self.format_arg(schema, {"token": data})

        self.assertEqual(args.to_serialized_data(), {"token": base64.b64decode(data).decode("utf-8")})

        # next_link pointing to a trusted data-plane endpoint, e.g., Microsoft Graph is accepted
        dataplane_next_link = "https://graph.microsoft.com/v1.0/users?$skiptoken=abc"
        data = _encode({"next_link": dataplane_next_link, "offset": 0})
        args = self.format_arg(schema, {"token": data})

        self.assertEqual(args.to_serialized_data(), {"token": base64.b64decode(data).decode("utf-8")})

        # next_link pointing to a lookalike host must be rejected (prefix-matching attack)
        spoofed_next_link = "https://management.azure.com.attacker/subscriptions/00000000-0000-0000-0000-000000000000/resources?api-version=2025-09-01"
        data = _encode({"next_link": spoofed_next_link, "offset": 0})
        with self.assertRaises(azclierror.InvalidArgumentValueError) as e:
            self.format_arg(schema, {"token": data})

        self.assertEqual(str(e.exception), "InvalidArgumentValue: --next-token: `next_link` '{}' is not a valid endpoint.".format(spoofed_next_link))

        # next_link using a userinfo trick where the real host is the attacker must be rejected
        userinfo_next_link = "https://management.azure.com@attacker/subscriptions/00000000-0000-0000-0000-000000000000/resources?api-version=2025-09-01"
        data = _encode({"next_link": userinfo_next_link, "offset": 0})
        with self.assertRaises(azclierror.InvalidArgumentValueError) as e:
            self.format_arg(schema, {"token": data})

        self.assertEqual(str(e.exception), "InvalidArgumentValue: --next-token: `next_link` '{}' is not a valid endpoint.".format(userinfo_next_link))




# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import json
from knack.util import CLIError
from azure.cli.core.util import send_raw_request
from azure.cli.core.commands.client_factory import get_subscription_id
from ._utils import validate_premium_registry


NETWORK_RULE_NOT_SUPPORTED = 'Network rules are only supported for managed registries in Premium SKU.'
# TODO: The networkRuleSet property was unintentionally removed from preview APIs.
# This was not previously exposed, because of multi-api support in the Python SDK.
# The Python SDK no longer supports multi-api, highlighting this issue.
# Until the next preview API release is deployed, this code uses direct REST calls to get and update
# the registry's networkRuleSet. Once the next preview API is released, this code should be updated
# to use the SDK methods to get and update the networkRuleSet, and the api version should be updated
# to reflect the latest API version, rather than hard coded, as it currently is below.
API_VERSION = "2021-08-01-preview"


def _get_registry_url(cli_ctx, resource_group_name, registry_name):
    """Build the REST API URL for a registry."""
    subscription_id = get_subscription_id(cli_ctx)
    return (
        "/subscriptions/{}/resourceGroups/{}/providers/Microsoft.ContainerRegistry/registries/{}?api-version={}"
        .format(subscription_id, resource_group_name, registry_name, API_VERSION)
    )


def _get_registry(cli_ctx, resource_group_name, registry_name):
    """Get registry using REST API."""
    url = _get_registry_url(cli_ctx, resource_group_name, registry_name)
    response = send_raw_request(cli_ctx, "GET", url)
    return response.json()


def _update_registry(cli_ctx, resource_group_name, registry_name, update_payload):
    """Update registry using REST API (PATCH)."""
    url = _get_registry_url(cli_ctx, resource_group_name, registry_name)
    response = send_raw_request(cli_ctx, "PATCH", url, body=json.dumps(update_payload))
    return response.json()


def _format_registry_response(response):
    """Format the registry REST response for CLI output."""
    properties = response.get('properties', {})
    network_rule_set = properties.get('networkRuleSet', {})

    virtual_network_rules = [
        {'virtualNetworkResourceId': rule.get('id'), 'action': rule.get('action', 'Allow')}
        for rule in (network_rule_set.get('virtualNetworkRules') or [])
    ]
    ip_rules = [
        {'ipAddressOrRange': rule.get('value') or rule.get('ipAddressOrRange'), 'action': rule.get('action', 'Allow')}
        for rule in (network_rule_set.get('ipRules') or [])
    ]

    return {
        'name': response.get('name'),
        'provisioningState': properties.get('provisioningState'),
        'networkRuleSet': {
            'defaultAction': network_rule_set.get('defaultAction'),
            'virtualNetworkRules': virtual_network_rules,
            'ipRules': ip_rules,
        },
    }


def acr_network_rule_list(cmd, registry_name, resource_group_name=None):
    _, resource_group_name = validate_premium_registry(
        cmd, registry_name, resource_group_name, NETWORK_RULE_NOT_SUPPORTED)

    registry = _get_registry(cmd.cli_ctx, resource_group_name, registry_name)
    network_rule_set = _format_registry_response(registry)['networkRuleSet']
    return {'virtualNetworkRules': network_rule_set['virtualNetworkRules'], 'ipRules': network_rule_set['ipRules']}


def acr_network_rule_add(cmd,
                         registry_name,
                         subnet=None,
                         vnet_name=None,
                         ip_address=None,
                         resource_group_name=None):
    _, resource_group_name = validate_premium_registry(
        cmd, registry_name, resource_group_name, NETWORK_RULE_NOT_SUPPORTED)

    registry = _get_registry(cmd.cli_ctx, resource_group_name, registry_name)
    rules = registry.get('properties', {}).get('networkRuleSet', {})

    if subnet or vnet_name:
        virtual_network_rules = list(rules.get('virtualNetworkRules') or [])
        subnet_id = _validate_subnet(cmd.cli_ctx, subnet, vnet_name, resource_group_name)
        virtual_network_rules.append({'id': subnet_id, 'action': 'Allow'})
        rules['virtualNetworkRules'] = virtual_network_rules

    if ip_address:
        ip_rules = list(rules.get('ipRules') or [])
        ip_rules.append({'value': ip_address, 'action': 'Allow'})
        rules['ipRules'] = ip_rules

    response = _update_registry(cmd.cli_ctx, resource_group_name, registry_name,
                                {'properties': {'networkRuleSet': rules}})
    return _format_registry_response(response)


def acr_network_rule_remove(cmd,
                            registry_name,
                            subnet=None,
                            vnet_name=None,
                            ip_address=None,
                            resource_group_name=None):
    _, resource_group_name = validate_premium_registry(
        cmd, registry_name, resource_group_name, NETWORK_RULE_NOT_SUPPORTED)

    registry = _get_registry(cmd.cli_ctx, resource_group_name, registry_name)
    rules = registry.get('properties', {}).get('networkRuleSet', {})

    if subnet or vnet_name:
        virtual_network_rules = list(rules.get('virtualNetworkRules') or [])
        subnet_id = _validate_subnet(cmd.cli_ctx, subnet, vnet_name, resource_group_name).lower()
        rules['virtualNetworkRules'] = [
            x for x in virtual_network_rules if x.get('id', '').lower() != subnet_id
        ]

    if ip_address:
        ip_rules = list(rules.get('ipRules') or [])
        rules['ipRules'] = [
            x for x in ip_rules
            if (x.get('value') or x.get('ipAddressOrRange')) != ip_address
        ]

    response = _update_registry(cmd.cli_ctx, resource_group_name, registry_name,
                                {'properties': {'networkRuleSet': rules}})
    return _format_registry_response(response)


def _validate_subnet(cli_ctx, subnet, vnet_name, resource_group_name):
    from azure.mgmt.core.tools import is_valid_resource_id
    subnet_is_id = is_valid_resource_id(subnet)

    if subnet_is_id and not vnet_name:
        return subnet
    if subnet and not subnet_is_id and vnet_name:
        from azure.mgmt.core.tools import resource_id
        return resource_id(
            subscription=get_subscription_id(cli_ctx),
            resource_group=resource_group_name,
            namespace='Microsoft.Network',
            type='virtualNetworks',
            name=vnet_name,
            child_type_1='subnets',
            child_name_1=subnet)
    raise CLIError('Usage error: [--subnet ID | --subnet NAME --vnet-name NAME]')



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

# pylint: disable=unused-argument, line-too-long
from azure.cli.core.commands.client_factory import get_subscription_id
from azure.cli.core.util import CLIError
from knack.log import get_logger
from .._client_factory import (
    cf_postgres_flexible_tuning_options,
    get_postgresql_flexible_management_client)
from ..utils._flexible_server_location_capabilities_util import (
    get_postgres_location_capability_info,
    get_postgres_server_capability_info)
from ..utils._util import get_autonomous_tuning_settings_map
from ..utils.validators import validate_resource_group
from .parameter_commands import _update_parameters, flexible_parameter_update

logger = get_logger(__name__)


def index_tuning_update(cmd, client, resource_group_name, server_name, index_tuning_enabled):
    validate_resource_group(resource_group_name)
    source = "user-override"

    if index_tuning_enabled == "True":
        list_capability_info = get_postgres_server_capability_info(cmd, resource_group_name, server_name, is_offer_restriction_check_required=True)
        autonomous_tuning_supported = list_capability_info['autonomous_tuning_supported']
        if not autonomous_tuning_supported:
            raise CLIError("Index tuning is not supported for the server.")

        logger.warning("Enabling index tuning for the server.")
        configuration_name = "index_tuning.mode"
        value = "report"
        _update_parameters(cmd, client, server_name, configuration_name, resource_group_name, source, value)
        configuration_name = "pg_qs.query_capture_mode"
        query_capture_mode_configuration = client.get(resource_group_name, server_name, configuration_name)

        if query_capture_mode_configuration.value.lower() == "none":
            value = "all"
            _update_parameters(cmd, client, server_name, configuration_name, resource_group_name, source, value)
        logger.warning("Index tuning is enabled for the server.")
    else:
        logger.warning("Disabling index tuning for the server.")
        configuration_name = "index_tuning.mode"
        value = "off"
        _update_parameters(cmd, client, server_name, configuration_name, resource_group_name, source, value)
        logger.warning("Index tuning is disabled for the server.")


def index_tuning_show(client, resource_group_name, server_name):
    validate_resource_group(resource_group_name)
    index_tuning_configuration = client.get(resource_group_name, server_name, "index_tuning.mode")
    query_capture_mode_configuration = client.get(resource_group_name, server_name, "pg_qs.query_capture_mode")

    if index_tuning_configuration.value.lower() == "report" and query_capture_mode_configuration.value.lower() != "none":
        logger.warning("Index tuning is enabled for the server.")
    else:
        logger.warning("Index tuning is disabled for the server.")


def index_tuning_settings_list(cmd, client, resource_group_name, server_name):
    validate_resource_group(resource_group_name)
    index_tuning_configurations_map_values = get_autonomous_tuning_settings_map().values()
    configurations_list = client.list_by_server(resource_group_name, server_name)

    # Filter the list based on the values in the dictionary
    index_tuning_settings = [setting for setting in configurations_list if setting.name in index_tuning_configurations_map_values]

    return index_tuning_settings


def index_tuning_settings_get(cmd, client, resource_group_name, server_name, setting_name):
    validate_resource_group(resource_group_name)
    index_tuning_configurations_map = get_autonomous_tuning_settings_map()
    index_tuning_configuration_name = index_tuning_configurations_map[setting_name]

    return client.get(
        resource_group_name=resource_group_name,
        server_name=server_name,
        configuration_name=index_tuning_configuration_name)


def index_tuning_settings_set(client, resource_group_name, server_name, setting_name, value=None):
    source = "user-override" if value else None
    tuning_settings = get_autonomous_tuning_settings_map()
    configuration_name = tuning_settings[setting_name]
    return flexible_parameter_update(client, server_name, configuration_name, resource_group_name, source, value)


def index_tuning_recommendations_list(cmd, resource_group_name, server_name, recommendation_type=None):
    validate_resource_group(resource_group_name)
    tuning_options_client = cf_postgres_flexible_tuning_options(cmd.cli_ctx, None)

    return tuning_options_client.list_recommendations(
        resource_group_name=resource_group_name,
        server_name=server_name,
        tuning_option="index",
        recommendation_type=recommendation_type
    )


def autonomous_tuning_update(cmd, client, resource_group_name, server_name, autonomous_tuning_enabled):
    validate_resource_group(resource_group_name)
    source = "user-override"

    if autonomous_tuning_enabled == "True":
        subscription = get_subscription_id(cmd.cli_ctx)
        postgres_source_client = get_postgresql_flexible_management_client(cmd.cli_ctx, subscription)
        source_server_object = postgres_source_client.servers.get(resource_group_name, server_name)
        location = ''.join(source_server_object.location.lower().split())
        list_location_capability_info = get_postgres_location_capability_info(cmd, location, is_offer_restriction_check_required=True)
        autonomous_tuning_supported = list_location_capability_info['autonomous_tuning_supported']
        if not autonomous_tuning_supported:
            raise CLIError("Autonomous tuning is not supported for the server.")

        logger.warning("Enabling autonomous tuning for the server.")
        configuration_name = "index_tuning.mode"
        value = "report"
        _update_parameters(cmd, client, server_name, configuration_name, resource_group_name, source, value)
        configuration_name = "pg_qs.query_capture_mode"
        query_capture_mode_configuration = client.get(resource_group_name, server_name, configuration_name)

        if query_capture_mode_configuration.value.lower() == "none":
            value = "all"
            _update_parameters(cmd, client, server_name, configuration_name, resource_group_name, source, value)
        logger.warning("Autonomous tuning is enabled for the server.")
    else:
        logger.warning("Disabling autonomous tuning for the server.")
        configuration_name = "index_tuning.mode"
        value = "off"
        _update_parameters(cmd, client, server_name, configuration_name, resource_group_name, source, value)
        logger.warning("Autonomous tuning is disabled for the server.")


def autonomous_tuning_show(client, resource_group_name, server_name):
    validate_resource_group(resource_group_name)
    autonomous_tuning_configuration = client.get(resource_group_name, server_name, "index_tuning.mode")
    query_capture_mode_configuration = client.get(resource_group_name, server_name, "pg_qs.query_capture_mode")

    if autonomous_tuning_configuration.value.lower() == "report" and query_capture_mode_configuration.value.lower() != "none":
        logger.warning("Autonomous tuning is enabled for the server.")
    else:
        logger.warning("Autonomous tuning is disabled for the server.")


def autonomous_tuning_settings_list(cmd, client, resource_group_name, server_name):
    validate_resource_group(resource_group_name)
    autonomous_tuning_configurations_map_values = get_autonomous_tuning_settings_map().values()
    configurations_list = client.list_by_server(resource_group_name, server_name)

    # Filter the list based on the values in the dictionary
    autonomous_tuning_settings = [setting for setting in configurations_list if setting.name in autonomous_tuning_configurations_map_values]

    return autonomous_tuning_settings


def autonomous_tuning_settings_get(cmd, client, resource_group_name, server_name, setting_name):
    validate_resource_group(resource_group_name)
    autonomous_tuning_configurations_map = get_autonomous_tuning_settings_map()
    autonomous_tuning_configuration_name = autonomous_tuning_configurations_map[setting_name]

    return client.get(
        resource_group_name=resource_group_name,
        server_name=server_name,
        configuration_name=autonomous_tuning_configuration_name)


def autonomous_tuning_settings_set(client, resource_group_name, server_name, setting_name, value=None):
    source = "user-override" if value else None
    tuning_settings = get_autonomous_tuning_settings_map()
    configuration_name = tuning_settings[setting_name]
    return flexible_parameter_update(client, server_name, configuration_name, resource_group_name, source, value)


def autonomous_tuning_index_recommendations_list(cmd, resource_group_name, server_name, recommendation_type=None):
    validate_resource_group(resource_group_name)
    tuning_options_client = cf_postgres_flexible_tuning_options(cmd.cli_ctx, None)

    return tuning_options_client.list_recommendations(
        resource_group_name=resource_group_name,
        server_name=server_name,
        tuning_option="index",
        recommendation_type=recommendation_type
    )


def autonomous_tuning_table_recommendations_list(cmd, resource_group_name, server_name, recommendation_type=None):
    validate_resource_group(resource_group_name)
    tuning_options_client = cf_postgres_flexible_tuning_options(cmd.cli_ctx, None)

    return tuning_options_client.list_recommendations(
        resource_group_name=resource_group_name,
        server_name=server_name,
        tuning_option="table",
        recommendation_type=recommendation_type
    )



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------


def create_log_profile_operations(client, name, location, locations, categories, days, enabled, tags=None,
                                  storage_account_id=None, service_bus_rule_id=None):
    from azure.mgmt.monitor.models import RetentionPolicy, LogProfileResource
    parameters = LogProfileResource(location=location, locations=locations, categories=categories,
                                    retention_policy=RetentionPolicy(days=days, enabled=enabled),
                                    storage_account_id=storage_account_id, service_bus_rule_id=service_bus_rule_id,
                                    tags=tags)
    return client.create_or_update(log_profile_name=name, parameters=parameters)



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import sys
from subprocess import check_output, STDOUT, CalledProcessError

def mean(data):
    """Return the sample arithmetic mean of data."""
    n = len(data)
    if n < 1:
        raise ValueError('len < 1')
    return sum(data)/float(n)


def sq_deviation(data):
    """Return sum of square deviations of sequence data."""
    c = mean(data)
    return sum((x-c)**2 for x in data)


def pstdev(data):
    """Calculates the population standard deviation."""
    n = len(data)
    if n < 2:
        raise ValueError('len < 2')
    ss = sq_deviation(data)
    return (ss/n) ** 0.5


def scenario(command, loop=10):
    real = []
    user = []
    syst = []

    test_command = 'time -p ' + command
    for i in range(loop):
        try:
            lines = check_output([test_command], shell=True, stderr=STDOUT).split('\n')
        except CalledProcessError as e:
            lines = e.output.split('\n')
        real_time = float(lines[-4].split()[1])
        real.append(float(lines[-4].split()[1]))
        user.append(float(lines[-3].split()[1]))
        syst.append(float(lines[-2].split()[1]))
        sys.stdout.write('Loop {} => {} \n'.format(i, real_time))
        sys.stdout.flush()

    print('Command: {}'.format(command))
    print('Real: mean => {} \t pstdev => {}'.format(mean(real), pstdev(real)))
    print('User: mean => {} \t pstdev => {}'.format(mean(user), pstdev(user)))
    print('Syst: mean => {} \t pstdev => {}'.format(mean(syst), pstdev(syst)))
    print('')

scenario('az')
scenario('az cl')
scenario('az cloud')
scenario('az cloud list')
scenario('az cloud show --this-does-not-exist')



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
#
# Code generated by aaz-dev-tools
# --------------------------------------------------------------------------------------------

# pylint: skip-file
# flake8: noqa

from azure.cli.core.aaz import *


@register_command(
    "monitor metrics list-sub",
)
class ListSub(AAZCommand):
    """Lists the metric data for a subscription. Parameters can be specified on the body.
    """

    _aaz_info = {
        "version": "2023-10-01",
        "resources": [
            ["mgmt-plane", "/subscriptions/{}/providers/microsoft.insights/metrics", "2023-10-01"],
        ]
    }

    def _handler(self, command_args):
        super()._handler(command_args)
        self._execute_operations()
        return self._output()

    _args_schema = None

    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        if cls._args_schema is not None:
            return cls._args_schema
        cls._args_schema = super()._build_arguments_schema(*args, **kwargs)

        # define Arg Group ""

        _args_schema = cls._args_schema
        _args_schema.region = AAZStrArg(
            options=["--region"],
            help="The region where the metrics you want reside.",
            required=True,
        )

        # define Arg Group "Body"

        _args_schema = cls._args_schema
        _args_schema.aggregation = AAZStrArg(
            options=["--aggregation"],
            arg_group="Body",
            help="The list of aggregation types (comma separated) to retrieve.",
        )
        _args_schema.auto_adjust_timegrain = AAZBoolArg(
            options=["--auto-adjust-timegrain"],
            arg_group="Body",
            help="When set to true, if the timespan passed in is not supported by this metric, the API will return the result using the closest supported timespan. When set to false, an error is returned for invalid timespan parameters. Defaults to false.",
        )
        _args_schema.filter = AAZStrArg(
            options=["--filter"],
            arg_group="Body",
            help="The **$filter** is used to reduce the set of metric data returned.<br>Example:<br>Metric contains metadata A, B and C.<br>- Return all time series of C where A = a1 and B = b1 or b2<br>**$filter=A eq ‘a1’ and B eq ‘b1’ or B eq ‘b2’ and C eq ‘*’**<br>- Invalid variant:<br>**$filter=A eq ‘a1’ and B eq ‘b1’ and C eq ‘*’ or B = ‘b2’**<br>This is invalid because the logical or operator cannot separate two different metadata names.<br>- Return all time series where A = a1, B = b1 and C = c1:<br>**$filter=A eq ‘a1’ and B eq ‘b1’ and C eq ‘c1’**<br>- Return all time series where A = a1<br>**$filter=A eq ‘a1’ and B eq ‘*’ and C eq ‘*’**.",
        )
        _args_schema.interval = AAZStrArg(
            options=["--interval"],
            arg_group="Body",
            help="The interval (i.e. timegrain) of the query in ISO 8601 duration format. Defaults to PT1M. Special case for 'FULL' value that returns single datapoint for entire time span requested. *Examples: PT15M, PT1H, P1D, FULL*",
        )
        _args_schema.metricnames = AAZStrArg(
            options=["--metricnames"],
            arg_group="Body",
            help="The names of the metrics (comma separated) to retrieve.",
        )
        _args_schema.metricnamespace = AAZStrArg(
            options=["--metricnamespace"],
            arg_group="Body",
            help="Metric namespace where the metrics you want reside.",
        )
        _args_schema.order_by = AAZStrArg(
            options=["--order-by"],
            arg_group="Body",
            help="The aggregation to use for sorting results and the direction of the sort. Only one order can be specified. Examples: sum asc.",
        )
        _args_schema.result_type = AAZStrArg(
            options=["--result-type"],
            arg_group="Body",
            help="Reduces the set of data collected. The syntax allowed depends on the operation. See the operation's description for details.",
            enum={"Data": "Data", "Metadata": "Metadata"},
        )
        _args_schema.roll_up_by = AAZStrArg(
            options=["--roll-up-by"],
            arg_group="Body",
            help="Dimension name(s) to rollup results by. For example if you only want to see metric values with a filter like 'City eq Seattle or City eq Tacoma' but don't want to see separate values for each city, you can specify 'RollUpBy=City' to see the results for Seattle and Tacoma rolled up into one timeseries.",
        )
        _args_schema.timespan = AAZStrArg(
            options=["--timespan"],
            arg_group="Body",
            help="The timespan of the query. It is a string with the following format 'startDateTime_ISO/endDateTime_ISO'.",
        )
        _args_schema.top = AAZIntArg(
            options=["--top"],
            arg_group="Body",
            help="The maximum number of records to retrieve. Valid only if $filter is specified. Defaults to 10.",
        )
        _args_schema.validate_dimensions = AAZBoolArg(
            options=["--validate-dimensions"],
            arg_group="Body",
            help="When set to false, invalid filter parameter values will be ignored. When set to true, an error is returned for invalid filter parameters. Defaults to true.",
        )
        return cls._args_schema

    def _execute_operations(self):
        self.pre_operations()
        self.MetricsListAtSubscriptionScopePost(ctx=self.ctx)()
        self.post_operations()

    @register_callback
    def pre_operations(self):
        pass

    @register_callback
    def post_operations(self):
        pass

    def _output(self, *args, **kwargs):
        result = self.deserialize_output(self.ctx.vars.instance, client_flatten=True)
        return result

    class MetricsListAtSubscriptionScopePost(AAZHttpOperation):
        CLIENT_TYPE = "MgmtClient"

        def __call__(self, *args, **kwargs):
            request = self.make_request()
            session = self.client.send_request(request=request, stream=False, **kwargs)
            if session.http_response.status_code in [200]:
                return self.on_200(session)

            return self.on_error(session.http_response)

        @property
        def url(self):
            return self.client.format_url(
                "/subscriptions/{subscriptionId}/providers/Microsoft.Insights/metrics",
                **self.url_parameters
            )

        @property
        def method(self):
            return "POST"

        @property
        def error_format(self):
            return "MgmtErrorFormat"

        @property
        def url_parameters(self):
            parameters = {
                **self.serialize_url_param(
                    "subscriptionId", self.ctx.subscription_id,
                    required=True,
                ),
            }
            return parameters

        @property
        def query_parameters(self):
            parameters = {
                **self.serialize_query_param(
                    "region", self.ctx.args.region,
                    required=True,
                ),
                **self.serialize_query_param(
                    "api-version", "2023-10-01",
                    required=True,
                ),
            }
            return parameters

        @property
        def header_parameters(self):
            parameters = {
                **self.serialize_header_param(
                    "Content-Type", "application/json",
                ),
                **self.serialize_header_param(
                    "Accept", "application/json",
                ),
            }
            return parameters

        @property
        def content(self):
            _content_value, _builder = self.new_content_builder(
                self.ctx.args,
                typ=AAZObjectType,
                typ_kwargs={"flags": {"client_flatten": True}}
            )
            _builder.set_prop("aggregation", AAZStrType, ".aggregation")
            _builder.set_prop("autoAdjustTimegrain", AAZBoolType, ".auto_adjust_timegrain")
            _builder.set_prop("filter", AAZStrType, ".filter")
            _builder.set_prop("interval", AAZStrType, ".interval")
            _builder.set_prop("metricNames", AAZStrType, ".metricnames")
            _builder.set_prop("metricNamespace", AAZStrType, ".metricnamespace")
            _builder.set_prop("orderBy", AAZStrType, ".order_by")
            _builder.set_prop("resultType", AAZStrType, ".result_type")
            _builder.set_prop("rollUpBy", AAZStrType, ".roll_up_by")
            _builder.set_prop("timespan", AAZStrType, ".timespan")
            _builder.set_prop("top", AAZIntType, ".top")
            _builder.set_prop("validateDimensions", AAZBoolType, ".validate_dimensions")

            return self.serialize_content(_content_value)

        def on_200(self, session):
            data = self.deserialize_http_content(session)
            self.ctx.set_var(
                "instance",
                data,
                schema_builder=self._build_schema_on_200
            )

        _schema_on_200 = None

        @classmethod
        def _build_schema_on_200(cls):
            if cls._schema_on_200 is not None:
                return cls._schema_on_200

            cls._schema_on_200 = AAZObjectType()

            _schema_on_200 = cls._schema_on_200
            _schema_on_200.cost = AAZIntType()
            _schema_on_200.interval = AAZStrType()
            _schema_on_200.namespace = AAZStrType()
            _schema_on_200.resourceregion = AAZStrType()
            _schema_on_200.timespan = AAZStrType(
                flags={"required": True},
            )
            _schema_on_200.value = AAZListType(
                flags={"required": True},
            )

            value = cls._schema_on_200.value
            value.Element = AAZObjectType()

            _element = cls._schema_on_200.value.Element
            _element.display_description = AAZStrType(
                serialized_name="displayDescription",
            )
            _element.error_code = AAZStrType(
                serialized_name="errorCode",
            )
            _element.error_message = AAZStrType(
                serialized_name="errorMessage",
            )
            _element.id = AAZStrType(
                flags={"required": True},
            )
            _element.name = AAZObjectType(
                flags={"required": True},
            )
            _ListSubHelper._build_schema_localizable_string_read(_element.name)
            _element.timeseries = AAZListType(
                flags={"required": True},
            )
            _element.type = AAZStrType(
                flags={"required": True},
            )
            _element.unit = AAZStrType(
                flags={"required": True},
            )

            timeseries = cls._schema_on_200.value.Element.timeseries
            timeseries.Element = AAZObjectType()

            _element = cls._schema_on_200.value.Element.timeseries.Element
            _element.data = AAZListType()
            _element.metadatavalues = AAZListType()

            data = cls._schema_on_200.value.Element.timeseries.Element.data
            data.Element = AAZObjectType()

            _element = cls._schema_on_200.value.Element.timeseries.Element.data.Element
            _element.average = AAZFloatType()
            _element.count = AAZFloatType()
            _element.maximum = AAZFloatType()
            _element.minimum = AAZFloatType()
            _element.time_stamp = AAZStrType(
                serialized_name="timeStamp",
                flags={"required": True},
            )
            _element.total = AAZFloatType()

            metadatavalues = cls._schema_on_200.value.Element.timeseries.Element.metadatavalues
            metadatavalues.Element = AAZObjectType()

            _element = cls._schema_on_200.value.Element.timeseries.Element.metadatavalues.Element
            _element.name = AAZObjectType()
            _ListSubHelper._build_schema_localizable_string_read(_element.name)
            _element.value = AAZStrType()

            return cls._schema_on_200


class _ListSubHelper:
    """Helper class for ListSub"""

    _schema_localizable_string_read = None

    @classmethod
    def _build_schema_localizable_string_read(cls, _schema):
        if cls._schema_localizable_string_read is not None:
            _schema.localized_value = cls._schema_localizable_string_read.localized_value
            _schema.value = cls._schema_localizable_string_read.value
            return

        cls._schema_localizable_string_read = _schema_localizable_string_read = AAZObjectType()

        localizable_string_read = _schema_localizable_string_read
        localizable_string_read.localized_value = AAZStrType(
            serialized_name="localizedValue",
        )
        localizable_string_read.value = AAZStrType(
            flags={"required": True},
        )

        _schema.localized_value = cls._schema_localizable_string_read.localized_value
        _schema.value = cls._schema_localizable_string_read.value


__all__ = ["ListSub"]



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

# pylint: disable=protected-access

from azure.cli.command_modules.monitor.aaz.latest.monitor.log_analytics.workspace.linked_storage._create \
    import Create as _WorkspaceLinkedStorageAccountCreate


class WorkspaceLinkedStorageAccountCreate(_WorkspaceLinkedStorageAccountCreate):

    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        from azure.cli.core.aaz import AAZResourceIdArg, AAZResourceIdArgFormat
        cls._args_schema = super()._build_arguments_schema(*args, **kwargs)

        storage_accounts = cls._args_schema.storage_accounts
        storage_accounts._element = AAZResourceIdArg(fmt=AAZResourceIdArgFormat(
            template='/subscriptions/{subscription}/resourceGroups/{resource_group}/'
                     'providers/Microsoft.Storage/storageAccounts/{}'))
        return cls._args_schema



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
#
# Code generated by aaz-dev-tools
# --------------------------------------------------------------------------------------------

# pylint: skip-file
# flake8: noqa

from azure.cli.core.aaz import *


@register_command(
    "lab vm show",
    is_preview=True,
)
class Show(AAZCommand):
    """Show virtual machine.
    """

    _aaz_info = {
        "version": "2018-09-15",
        "resources": [
            ["mgmt-plane", "/subscriptions/{}/resourcegroups/{}/providers/microsoft.devtestlab/labs/{}/virtualmachines/{}", "2018-09-15"],
        ]
    }

    def _handler(self, command_args):
        super()._handler(command_args)
        self._execute_operations()
        return self._output()

    _args_schema = None

    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        if cls._args_schema is not None:
            return cls._args_schema
        cls._args_schema = super()._build_arguments_schema(*args, **kwargs)

        # define Arg Group ""

        _args_schema = cls._args_schema
        _args_schema.lab_name = AAZStrArg(
            options=["--lab-name"],
            help="The name of the lab.",
            required=True,
            id_part="name",
        )
        _args_schema.name = AAZStrArg(
            options=["--name"],
            help="The name of the virtual machine.",
            required=True,
            id_part="child_name_1",
        )
        _args_schema.resource_group = AAZResourceGroupNameArg(
            required=True,
        )
        _args_schema.expand = AAZStrArg(
            options=["--expand"],
            help="Specify the $expand query. Example: '\"properties($expand=artifacts,computeVm,networkInterface,applicableSchedule)\"'",
        )
        return cls._args_schema

    def _execute_operations(self):
        self.pre_operations()
        self.VirtualMachinesGet(ctx=self.ctx)()
        self.post_operations()

    @register_callback
    def pre_operations(self):
        pass

    @register_callback
    def post_operations(self):
        pass

    def _output(self, *args, **kwargs):
        result = self.deserialize_output(self.ctx.vars.instance, client_flatten=True)
        return result

    class VirtualMachinesGet(AAZHttpOperation):
        CLIENT_TYPE = "MgmtClient"

        def __call__(self, *args, **kwargs):
            request = self.make_request()
            session = self.client.send_request(request=request, stream=False, **kwargs)
            if session.http_response.status_code in [200]:
                return self.on_200(session)

            return self.on_error(session.http_response)

        @property
        def url(self):
            return self.client.format_url(
                "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.DevTestLab/labs/{labName}/virtualmachines/{name}",
                **self.url_parameters
            )

        @property
        def method(self):
            return "GET"

        @property
        def error_format(self):
            return "ODataV4Format"

        @property
        def url_parameters(self):
            parameters = {
                **self.serialize_url_param(
                    "labName", self.ctx.args.lab_name,
                    required=True,
                ),
                **self.serialize_url_param(
                    "name", self.ctx.args.name,
                    required=True,
                ),
                **self.serialize_url_param(
                    "resourceGroupName", self.ctx.args.resource_group,
                    required=True,
                ),
                **self.serialize_url_param(
                    "subscriptionId", self.ctx.subscription_id,
                    required=True,
                ),
            }
            return parameters

        @property
        def query_parameters(self):
            parameters = {
                **self.serialize_query_param(
                    "$expand", self.ctx.args.expand,
                ),
                **self.serialize_query_param(
                    "api-version", "2018-09-15",
                    required=True,
                ),
            }
            return parameters

        @property
        def header_parameters(self):
            parameters = {
                **self.serialize_header_param(
                    "Accept", "application/json",
                ),
            }
            return parameters

        def on_200(self, session):
            data = self.deserialize_http_content(session)
            self.ctx.set_var(
                "instance",
                data,
                schema_builder=self._build_schema_on_200
            )

        _schema_on_200 = None

        @classmethod
        def _build_schema_on_200(cls):
            if cls._schema_on_200 is not None:
                return cls._schema_on_200

            cls._schema_on_200 = AAZObjectType()

            _schema_on_200 = cls._schema_on_200
            _schema_on_200.id = AAZStrType(
                flags={"read_only": True},
            )
            _schema_on_200.location = AAZStrType()
            _schema_on_200.name = AAZStrType(
                flags={"read_only": True},
            )
            _schema_on_200.properties = AAZObjectType(
                flags={"required": True, "client_flatten": True},
            )
            _schema_on_200.tags = AAZDictType()
            _schema_on_200.type = AAZStrType(
                flags={"read_only": True},
            )

            properties = cls._schema_on_200.properties
            properties.allow_claim = AAZBoolType(
                serialized_name="allowClaim",
            )
            properties.applicable_schedule = AAZObjectType(
                serialized_name="applicableSchedule",
                flags={"read_only": True},
            )
            properties.artifact_deployment_status = AAZObjectType(
                serialized_name="artifactDeploymentStatus",
                flags={"read_only": True},
            )
            properties.artifacts = AAZListType()
            properties.compute_id = AAZStrType(
                serialized_name="computeId",
                flags={"read_only": True},
            )
            properties.compute_vm = AAZObjectType(
                serialized_name="computeVm",
                flags={"read_only": True},
            )
            properties.created_by_user = AAZStrType(
                serialized_name="createdByUser",
                flags={"read_only": True},
            )
            properties.created_by_user_id = AAZStrType(
                serialized_name="createdByUserId",
                flags={"read_only": True},
            )
            properties.created_date = AAZStrType(
                serialized_name="createdDate",
            )
            properties.custom_image_id = AAZStrType(
                serialized_name="customImageId",
            )
            properties.data_disk_parameters = AAZListType(
                serialized_name="dataDiskParameters",
            )
            properties.disallow_public_ip_address = AAZBoolType(
                serialized_name="disallowPublicIpAddress",
            )
            properties.environment_id = AAZStrType(
                serialized_name="environmentId",
            )
            properties.expiration_date = AAZStrType(
                serialized_name="expirationDate",
            )
            properties.fqdn = AAZStrType(
                flags={"read_only": True},
            )
            properties.gallery_image_reference = AAZObjectType(
                serialized_name="galleryImageReference",
            )
            properties.is_authentication_with_ssh_key = AAZBoolType(
                serialized_name="isAuthenticationWithSshKey",
            )
            properties.lab_subnet_name = AAZStrType(
                serialized_name="labSubnetName",
            )
            properties.lab_virtual_network_id = AAZStrType(
                serialized_name="labVirtualNetworkId",
            )
            properties.last_known_power_state = AAZStrType(
                serialized_name="lastKnownPowerState",
                flags={"read_only": True},
            )
            properties.network_interface = AAZObjectType(
                serialized_name="networkInterface",
            )
            properties.notes = AAZStrType()
            properties.os_type = AAZStrType(
                serialized_name="osType",
                flags={"read_only": True},
            )
            properties.owner_object_id = AAZStrType(
                serialized_name="ownerObjectId",
            )
            properties.owner_user_principal_name = AAZStrType(
                serialized_name="ownerUserPrincipalName",
            )
            properties.password = AAZStrType(
                flags={"secret": True},
            )
            properties.plan_id = AAZStrType(
                serialized_name="planId",
            )
            properties.provisioning_state = AAZStrType(
                serialized_name="provisioningState",
                flags={"read_only": True},
            )
            properties.schedule_parameters = AAZListType(
                serialized_name="scheduleParameters",
            )
            properties.size = AAZStrType()
            properties.ssh_key = AAZStrType(
                serialized_name="sshKey",
                flags={"secret": True},
            )
            properties.storage_type = AAZStrType(
                serialized_name="storageType",
            )
            properties.unique_identifier = AAZStrType(
                serialized_name="uniqueIdentifier",
                flags={"read_only": True},
            )
            properties.user_name = AAZStrType(
                serialized_name="userName",
            )
            properties.virtual_machine_creation_source = AAZStrType(
                serialized_name="virtualMachineCreationSource",
                flags={"read_only": True},
            )

            applicable_schedule = cls._schema_on_200.properties.applicable_schedule
            applicable_schedule.id = AAZStrType(
                flags={"read_only": True},
            )
            applicable_schedule.location = AAZStrType()
            applicable_schedule.name = AAZStrType(
                flags={"read_only": True},
            )
            applicable_schedule.properties = AAZObjectType(
                flags={"required": True, "client_flatten": True},
            )
            applicable_schedule.tags = AAZDictType()
            applicable_schedule.type = AAZStrType(
                flags={"read_only": True},
            )

            properties = cls._schema_on_200.properties.applicable_schedule.properties
            properties.lab_vms_shutdown = AAZObjectType(
                serialized_name="labVmsShutdown",
            )
            _ShowHelper._build_schema_schedule_read(properties.lab_vms_shutdown)
            properties.lab_vms_startup = AAZObjectType(
                serialized_name="labVmsStartup",
            )
            _ShowHelper._build_schema_schedule_read(properties.lab_vms_startup)

            tags = cls._schema_on_200.properties.applicable_schedule.tags
            tags.Element = AAZStrType()

            artifact_deployment_status = cls._schema_on_200.properties.artifact_deployment_status
            artifact_deployment_status.artifacts_applied = AAZIntType(
                serialized_name="artifactsApplied",
            )
            artifact_deployment_status.deployment_status = AAZStrType(
                serialized_name="deploymentStatus",
            )
            artifact_deployment_status.total_artifacts = AAZIntType(
                serialized_name="totalArtifacts",
            )

            artifacts = cls._schema_on_200.properties.artifacts
            artifacts.Element = AAZObjectType()

            _element = cls._schema_on_200.properties.artifacts.Element
            _element.artifact_id = AAZStrType(
                serialized_name="artifactId",
            )
            _element.artifact_title = AAZStrType(
                serialized_name="artifactTitle",
            )
            _element.deployment_status_message = AAZStrType(
                serialized_name="deploymentStatusMessage",
            )
            _element.install_time = AAZStrType(
                serialized_name="installTime",
            )
            _element.parameters = AAZListType()
            _element.status = AAZStrType()
            _element.vm_extension_status_message = AAZStrType(
                serialized_name="vmExtensionStatusMessage",
            )

            parameters = cls._schema_on_200.properties.artifacts.Element.parameters
            parameters.Element = AAZObjectType()

            _element = cls._schema_on_200.properties.artifacts.Element.parameters.Element
            _element.name = AAZStrType()
            _element.value = AAZStrType()

            compute_vm = cls._schema_on_200.properties.compute_vm
            compute_vm.data_disk_ids = AAZListType(
                serialized_name="dataDiskIds",
            )
            compute_vm.data_disks = AAZListType(
                serialized_name="dataDisks",
            )
            compute_vm.network_interface_id = AAZStrType(
                serialized_name="networkInterfaceId",
            )
            compute_vm.os_disk_id = AAZStrType(
                serialized_name="osDiskId",
            )
            compute_vm.os_type = AAZStrType(
                serialized_name="osType",
            )
            compute_vm.statuses = AAZListType()
            compute_vm.vm_size = AAZStrType(
                serialized_name="vmSize",
            )

            data_disk_ids = cls._schema_on_200.properties.compute_vm.data_disk_ids
            data_disk_ids.Element = AAZStrType()

            data_disks = cls._schema_on_200.properties.compute_vm.data_disks
            data_disks.Element = AAZObjectType()

            _element = cls._schema_on_200.properties.compute_vm.data_disks.Element
            _element.disk_size_gi_b = AAZIntType(
                serialized_name="diskSizeGiB",
            )
            _element.disk_uri = AAZStrType(
                serialized_name="diskUri",
            )
            _element.managed_disk_id = AAZStrType(
                serialized_name="managedDiskId",
            )
            _element.name = AAZStrType()

            statuses = cls._schema_on_200.properties.compute_vm.statuses
            statuses.Element = AAZObjectType()

            _element = cls._schema_on_200.properties.compute_vm.statuses.Element
            _element.code = AAZStrType()
            _element.display_status = AAZStrType(
                serialized_name="displayStatus",
            )
            _element.message = AAZStrType()

            data_disk_parameters = cls._schema_on_200.properties.data_disk_parameters
            data_disk_parameters.Element = AAZObjectType()

            _element = cls._schema_on_200.properties.data_disk_parameters.Element
            _element.attach_new_data_disk_options = AAZObjectType(
                serialized_name="attachNewDataDiskOptions",
            )
            _element.existing_lab_disk_id = AAZStrType(
                serialized_name="existingLabDiskId",
            )
            _element.host_caching = AAZStrType(
                serialized_name="hostCaching",
            )

            attach_new_data_disk_options = cls._schema_on_200.properties.data_disk_parameters.Element.attach_new_data_disk_options
            attach_new_data_disk_options.disk_name = AAZStrType(
                serialized_name="diskName",
            )
            attach_new_data_disk_options.disk_size_gi_b = AAZIntType(
                serialized_name="diskSizeGiB",
            )
            attach_new_data_disk_options.disk_type = AAZStrType(
                serialized_name="diskType",
            )

            gallery_image_reference = cls._schema_on_200.properties.gallery_image_reference
            gallery_image_reference.offer = AAZStrType()
            gallery_image_reference.os_type = AAZStrType(
                serialized_name="osType",
            )
            gallery_image_reference.publisher = AAZStrType()
            gallery_image_reference.sku = AAZStrType()
            gallery_image_reference.version = AAZStrType()

            network_interface = cls._schema_on_200.properties.network_interface
            network_interface.dns_name = AAZStrType(
                serialized_name="dnsName",
            )
            network_interface.private_ip_address = AAZStrType(
                serialized_name="privateIpAddress",
            )
            network_interface.public_ip_address = AAZStrType(
                serialized_name="publicIpAddress",
            )
            network_interface.public_ip_address_id = AAZStrType(
                serialized_name="publicIpAddressId",
            )
            network_interface.rdp_authority = AAZStrType(
                serialized_name="rdpAuthority",
            )
            network_interface.shared_public_ip_address_configuration = AAZObjectType(
                serialized_name="sharedPublicIpAddressConfiguration",
            )
            network_interface.ssh_authority = AAZStrType(
                serialized_name="sshAuthority",
            )
            network_interface.subnet_id = AAZStrType(
                serialized_name="subnetId",
            )
            network_interface.virtual_network_id = AAZStrType(
                serialized_name="virtualNetworkId",
            )

            shared_public_ip_address_configuration = cls._schema_on_200.properties.network_interface.shared_public_ip_address_configuration
            shared_public_ip_address_configuration.inbound_nat_rules = AAZListType(
                serialized_name="inboundNatRules",
            )

            inbound_nat_rules = cls._schema_on_200.properties.network_interface.shared_public_ip_address_configuration.inbound_nat_rules
            inbound_nat_rules.Element = AAZObjectType()

            _element = cls._schema_on_200.properties.network_interface.shared_public_ip_address_configuration.inbound_nat_rules.Element
            _element.backend_port = AAZIntType(
                serialized_name="backendPort",
            )
            _element.frontend_port = AAZIntType(
                serialized_name="frontendPort",
            )
            _element.transport_protocol = AAZStrType(
                serialized_name="transportProtocol",
            )

            schedule_parameters = cls._schema_on_200.properties.schedule_parameters
            schedule_parameters.Element = AAZObjectType()

            _element = cls._schema_on_200.properties.schedule_parameters.Element
            _element.location = AAZStrType(
                flags={"read_only": True},
            )
            _element.name = AAZStrType()
            _element.properties = AAZObjectType(
                flags={"client_flatten": True},
            )
            _element.tags = AAZDictType()

            properties = cls._schema_on_200.properties.schedule_parameters.Element.properties
            properties.daily_recurrence = AAZObjectType(
                serialized_name="dailyRecurrence",
            )
            _ShowHelper._build_schema_day_details_read(properties.daily_recurrence)
            properties.hourly_recurrence = AAZObjectType(
                serialized_name="hourlyRecurrence",
            )
            _ShowHelper._build_schema_hour_details_read(properties.hourly_recurrence)
            properties.notification_settings = AAZObjectType(
                serialized_name="notificationSettings",
            )
            _ShowHelper._build_schema_notification_settings_read(properties.notification_settings)
            properties.status = AAZStrType()
            properties.target_resource_id = AAZStrType(
                serialized_name="targetResourceId",
            )
            properties.task_type = AAZStrType(
                serialized_name="taskType",
            )
            properties.time_zone_id = AAZStrType(
                serialized_name="timeZoneId",
            )
            properties.weekly_recurrence = AAZObjectType(
                serialized_name="weeklyRecurrence",
            )
            _ShowHelper._build_schema_week_details_read(properties.weekly_recurrence)

            tags = cls._schema_on_200.properties.schedule_parameters.Element.tags
            tags.Element = AAZStrType()

            tags = cls._schema_on_200.tags
            tags.Element = AAZStrType()

            return cls._schema_on_200


class _ShowHelper:
    """Helper class for Show"""

    _schema_day_details_read = None

    @classmethod
    def _build_schema_day_details_read(cls, _schema):
        if cls._schema_day_details_read is not None:
            _schema.time = cls._schema_day_details_read.time
            return

        cls._schema_day_details_read = _schema_day_details_read = AAZObjectType()

        day_details_read = _schema_day_details_read
        day_details_read.time = AAZStrType()

        _schema.time = cls._schema_day_details_read.time

    _schema_hour_details_read = None

    @classmethod
    def _build_schema_hour_details_read(cls, _schema):
        if cls._schema_hour_details_read is not None:
            _schema.minute = cls._schema_hour_details_read.minute
            return

        cls._schema_hour_details_read = _schema_hour_details_read = AAZObjectType()

        hour_details_read = _schema_hour_details_read
        hour_details_read.minute = AAZIntType()

        _schema.minute = cls._schema_hour_details_read.minute

    _schema_notification_settings_read = None

    @classmethod
    def _build_schema_notification_settings_read(cls, _schema):
        if cls._schema_notification_settings_read is not None:
            _schema.email_recipient = cls._schema_notification_settings_read.email_recipient
            _schema.notification_locale = cls._schema_notification_settings_read.notification_locale
            _schema.status = cls._schema_notification_settings_read.status
            _schema.time_in_minutes = cls._schema_notification_settings_read.time_in_minutes
            _schema.webhook_url = cls._schema_notification_settings_read.webhook_url
            return

        cls._schema_notification_settings_read = _schema_notification_settings_read = AAZObjectType()

        notification_settings_read = _schema_notification_settings_read
        notification_settings_read.email_recipient = AAZStrType(
            serialized_name="emailRecipient",
        )
        notification_settings_read.notification_locale = AAZStrType(
            serialized_name="notificationLocale",
        )
        notification_settings_read.status = AAZStrType()
        notification_settings_read.time_in_minutes = AAZIntType(
            serialized_name="timeInMinutes",
        )
        notification_settings_read.webhook_url = AAZStrType(
            serialized_name="webhookUrl",
        )

        _schema.email_recipient = cls._schema_notification_settings_read.email_recipient
        _schema.notification_locale = cls._schema_notification_settings_read.notification_locale
        _schema.status = cls._schema_notification_settings_read.status
        _schema.time_in_minutes = cls._schema_notification_settings_read.time_in_minutes
        _schema.webhook_url = cls._schema_notification_settings_read.webhook_url

    _schema_schedule_read = None

    @classmethod
    def _build_schema_schedule_read(cls, _schema):
        if cls._schema_schedule_read is not None:
            _schema.id = cls._schema_schedule_read.id
            _schema.location = cls._schema_schedule_read.location
            _schema.name = cls._schema_schedule_read.name
            _schema.properties = cls._schema_schedule_read.properties
            _schema.tags = cls._schema_schedule_read.tags
            _schema.type = cls._schema_schedule_read.type
            return

        cls._schema_schedule_read = _schema_schedule_read = AAZObjectType()

        schedule_read = _schema_schedule_read
        schedule_read.id = AAZStrType(
            flags={"read_only": True},
        )
        schedule_read.location = AAZStrType()
        schedule_read.name = AAZStrType(
            flags={"read_only": True},
        )
        schedule_read.properties = AAZObjectType(
            flags={"required": True, "client_flatten": True},
        )
        schedule_read.tags = AAZDictType()
        schedule_read.type = AAZStrType(
            flags={"read_only": True},
        )

        properties = _schema_schedule_read.properties
        properties.created_date = AAZStrType(
            serialized_name="createdDate",
            flags={"read_only": True},
        )
        properties.daily_recurrence = AAZObjectType(
            serialized_name="dailyRecurrence",
        )
        cls._build_schema_day_details_read(properties.daily_recurrence)
        properties.hourly_recurrence = AAZObjectType(
            serialized_name="hourlyRecurrence",
        )
        cls._build_schema_hour_details_read(properties.hourly_recurrence)
        properties.notification_settings = AAZObjectType(
            serialized_name="notificationSettings",
        )
        cls._build_schema_notification_settings_read(properties.notification_settings)
        properties.provisioning_state = AAZStrType(
            serialized_name="provisioningState",
            flags={"read_only": True},
        )
        properties.status = AAZStrType()
        properties.target_resource_id = AAZStrType(
            serialized_name="targetResourceId",
        )
        properties.task_type = AAZStrType(
            serialized_name="taskType",
        )
        properties.time_zone_id = AAZStrType(
            serialized_name="timeZoneId",
        )
        properties.unique_identifier = AAZStrType(
            serialized_name="uniqueIdentifier",
            flags={"read_only": True},
        )
        properties.weekly_recurrence = AAZObjectType(
            serialized_name="weeklyRecurrence",
        )
        cls._build_schema_week_details_read(properties.weekly_recurrence)

        tags = _schema_schedule_read.tags
        tags.Element = AAZStrType()

        _schema.id = cls._schema_schedule_read.id
        _schema.location = cls._schema_schedule_read.location
        _schema.name = cls._schema_schedule_read.name
        _schema.properties = cls._schema_schedule_read.properties
        _schema.tags = cls._schema_schedule_read.tags
        _schema.type = cls._schema_schedule_read.type

    _schema_week_details_read = None

    @classmethod
    def _build_schema_week_details_read(cls, _schema):
        if cls._schema_week_details_read is not None:
            _schema.time = cls._schema_week_details_read.time
            _schema.weekdays = cls._schema_week_details_read.weekdays
            return

        cls._schema_week_details_read = _schema_week_details_read = AAZObjectType()

        week_details_read = _schema_week_details_read
        week_details_read.time = AAZStrType()
        week_details_read.weekdays = AAZListType()

        weekdays = _schema_week_details_read.weekdays
        weekdays.Element = AAZStrType()

        _schema.time = cls._schema_week_details_read.time
        _schema.weekdays = cls._schema_week_details_read.weekdays


__all__ = ["Show"]



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import json
from unittest import mock
import os
import unittest

import urllib.request as http_client_t
from urllib.error import HTTPError

from applicationinsights.channel import SynchronousSender


from azure.cli.telemetry.components.telemetry_client import CliTelemetryClient, _NoRetrySender

TEST_RESOURCE_FOLDER = os.path.join(os.path.dirname(__file__), 'resources')


class _TestSender(SynchronousSender):
    instances = []

    def __init__(self):
        super().__init__()
        _TestSender.instances.append(self)

        self.data = []

    def send(self, data_to_send):
        self.data.append(data_to_send)


class TestTelemetryClient(unittest.TestCase):
    def setUp(self):
        # load sample data
        self.sample_records = []
        for f in os.listdir(TEST_RESOURCE_FOLDER):
            with open(os.path.join(TEST_RESOURCE_FOLDER, f), mode='r') as fq:
                for line in fq.readlines():
                    self.sample_records.append(line[20:])

        del _TestSender.instances[:]

    def test_telemetry_client_without_flush(self):
        client = CliTelemetryClient(sender=_TestSender)

        self.assertEqual(0, len(_TestSender.instances))

        for r in self.sample_records[:10]:
            client.add(r)

        self.assertEqual(1, len(_TestSender.instances))
        sender = _TestSender.instances[0]

        self.assertEqual(0, len(sender.data))

        # flush is skipped because record collection size is small
        client.flush()
        self.assertEqual(0, len(sender.data))

        # force flush should send data set even it is smaller than the batch threshold
        # there should be 10 envelops in the batch of data
        client.flush(force=True)
        self.assertEqual(1, len(sender.data))
        self.assertEqual(10, len(sender.data[0]))

        # repeat flush should not duplicate data
        client.flush(force=True)
        self.assertEqual(1, len(sender.data))

        # default batch size is 100, ensure data is sent after accumulation
        del sender.data[:]
        count = 0
        for r in self.sample_records:
            client.add(r, flush=True)

            count += 1
            if not count % 100:
                self.assertEqual(1, len(sender.data))
                del sender.data[:]
            else:
                self.assertEqual(0, len(sender.data))


class TestNoRetrySender(unittest.TestCase):
    def setUp(self):
        # build some data in the form of envelops
        del _TestSender.instances[:]
        client = CliTelemetryClient(sender=_TestSender)
        with open(os.path.join(TEST_RESOURCE_FOLDER, 'cache'), mode='r') as fq:
            for line in fq.readlines()[:5]:
                client.add(line[20:], flush=True, force=True)
        self.sample_data = _TestSender.instances[0].data

    def test_limited_retry_sender(self):
        mock_url_open = mock.Mock()

        with mock.patch.object(http_client_t, 'urlopen', mock_url_open):
            sender = _NoRetrySender()
            sender.send(self.sample_data[0])

        mock_url_open.assert_called_once()
        args, kwargs = mock_url_open.call_args

        self.assertEqual(10, kwargs['timeout'])

        data = json.loads(args[0].data.decode('utf-8'))[0]
        self.assertEqual('Microsoft.ApplicationInsights.Event', data['name'])
        self.assertEqual('UserTask', data['data']['baseData']['properties']['Reserved.DataModel.EntityType'])
        self.assertEqual('azurecli', data['data']['baseData']['properties']['Reserved.DataModel.ProductName'])

    def test_limited_retry_sender_http_error(self):
        mock_url_open = mock.Mock()
        mock_url_open.side_effect = HTTPError('', 500, 'expected', [], None)

        with mock.patch.object(http_client_t, 'urlopen', mock_url_open):
            sender = _NoRetrySender()
            sender.send(self.sample_data[0])

        mock_url_open.assert_called_once()
        args, kwargs = mock_url_open.call_args

        self.assertEqual(10, kwargs['timeout'])

        data = json.loads(args[0].data.decode('utf-8'))[0]
        self.assertEqual('Microsoft.ApplicationInsights.Event', data['name'])
        self.assertEqual('UserTask', data['data']['baseData']['properties']['Reserved.DataModel.EntityType'])
        self.assertEqual('azurecli', data['data']['baseData']['properties']['Reserved.DataModel.ProductName'])

    def test_limited_retry_sender_os_error(self):
        mock_url_open = mock.Mock()
        mock_url_open.side_effect = OSError('error')

        with mock.patch.object(http_client_t, 'urlopen', mock_url_open):
            sender = _NoRetrySender()
            sender.send(self.sample_data[0])

        mock_url_open.assert_called_once()

    def test_limited_retry_sender_other_exception(self):
        mock_url_open = mock.Mock()
        mock_url_open.side_effect = ValueError('error')

        with mock.patch.object(http_client_t, 'urlopen', mock_url_open):
            sender = _NoRetrySender()
            sender.send(self.sample_data[0])

        mock_url_open.assert_called_once()


if __name__ == '__main__':
    unittest.main()



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------
# pylint: disable=protected-access

import argparse
from knack.util import CLIError


class AddArgs(argparse._AppendAction):
    def __call__(self, parser, namespace, values, option_string=None):
        action = self.get_action(values, option_string)
        super().__call__(parser, namespace, action, option_string)

    def get_action(self, values, option_string):  # pylint: disable=no-self-use
        try:
            properties = {}
            for (k, v) in (x.split('=', 1) for x in values):
                properties[k] = v
            return properties
        except ValueError:
            raise CLIError('usage error: {} [KEY=VALUE ...]'.format(option_string))



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
#
# Code generated by aaz-dev-tools
# --------------------------------------------------------------------------------------------

# pylint: skip-file
# flake8: noqa

from azure.cli.core.aaz import *


@register_command(
    "netappfiles volume ransomware-report clear-suspect",
)
class ClearSuspect(AAZCommand):
    """Clear ransomware suspects for the given Advanced Ransomware Protection (ARP) report. You should evaluate the report to determine whether the activity is acceptable (false positive) or whether an attack seems malicious.ARP creates snapshots named Anti_ransomware_backup when it detects a potential ransomware threat. You can use one of these ARP snapshots or another snapshot of your volume to restore data",

    :example: RansomwareReports_ClearSuspects
        az netappfiles volume ransomware-report clear-suspect --resource-group myRG --account-name account1 --pool-name pool1 --volume-name volume1 --ransomware-report-name ransomwareReport1 --resolution PotentialThreat --extensions "[.threat]"
    """

    _aaz_info = {
        "version": "2026-04-01",
        "resources": [
            ["mgmt-plane", "/subscriptions/{}/resourcegroups/{}/providers/microsoft.netapp/netappaccounts/{}/capacitypools/{}/volumes/{}/ransomwarereports/{}/clearsuspects", "2026-04-01"],
        ]
    }

    AZ_SUPPORT_NO_WAIT = True

    def _handler(self, command_args):
        super()._handler(command_args)
        return self.build_lro_poller(self._execute_operations, None)

    _args_schema = None

    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        if cls._args_schema is not None:
            return cls._args_schema
        cls._args_schema = super()._build_arguments_schema(*args, **kwargs)

        # define Arg Group ""

        _args_schema = cls._args_schema
        _args_schema.account_name = AAZStrArg(
            options=["-a", "--account-name"],
            help="The name of the NetApp account",
            required=True,
            id_part="name",
            fmt=AAZStrArgFormat(
                pattern="^[a-zA-Z0-9][a-zA-Z0-9\\-_]{0,127}$",
            ),
        )
        _args_schema.pool_name = AAZStrArg(
            options=["-p", "--pool-name"],
            help="The name of the capacity pool",
            required=True,
            id_part="child_name_1",
            fmt=AAZStrArgFormat(
                pattern="^[a-zA-Z0-9][a-zA-Z0-9\\-_]{0,63}$",
                max_length=64,
                min_length=1,
            ),
        )
        _args_schema.ransomware_report_name = AAZStrArg(
            options=["--ransomware-report-name"],
            help="The name of the ransomware report",
            required=True,
            id_part="child_name_3",
            fmt=AAZStrArgFormat(
                pattern="^[a-zA-Z0-9][a-zA-Z0-9\\-_]{0,63}$",
                max_length=64,
                min_length=1,
            ),
        )
        _args_schema.resource_group = AAZResourceGroupNameArg(
            required=True,
        )
        _args_schema.volume_name = AAZStrArg(
            options=["-v", "--volume-name"],
            help="The name of the volume",
            required=True,
            id_part="child_name_2",
            fmt=AAZStrArgFormat(
                pattern="^[a-zA-Z][a-zA-Z0-9\\-_]{0,63}$",
                max_length=64,
                min_length=1,
            ),
        )

        # define Arg Group "Body"

        _args_schema = cls._args_schema
        _args_schema.extensions = AAZListArg(
            options=["--extensions"],
            arg_group="Body",
            help="List of file extensions resolved (PotentialThreat or FalsePositive)",
            required=True,
            fmt=AAZListArgFormat(
                min_length=1,
            ),
        )
        _args_schema.resolution = AAZStrArg(
            options=["--resolution"],
            arg_group="Body",
            help="ARP report suspect resolution",
            required=True,
            enum={"FalsePositive": "FalsePositive", "PotentialThreat": "PotentialThreat"},
        )

        extensions = cls._args_schema.extensions
        extensions.Element = AAZStrArg(
            fmt=AAZStrArgFormat(
                max_length=1024,
                min_length=1,
            ),
        )
        return cls._args_schema

    def _execute_operations(self):
        self.pre_operations()
        yield self.RansomwareReportsClearSuspects(ctx=self.ctx)()
        self.post_operations()

    @register_callback
    def pre_operations(self):
        pass

    @register_callback
    def post_operations(self):
        pass

    class RansomwareReportsClearSuspects(AAZHttpOperation):
        CLIENT_TYPE = "MgmtClient"

        def __call__(self, *args, **kwargs):
            request = self.make_request()
            session = self.client.send_request(request=request, stream=False, **kwargs)
            if session.http_response.status_code in [202]:
                return self.client.build_lro_polling(
                    self.ctx.args.no_wait,
                    session,
                    None,
                    self.on_error,
                    lro_options={"final-state-via": "location"},
                    path_format_arguments=self.url_parameters,
                )

            return self.on_error(session.http_response)

        @property
        def url(self):
            return self.client.format_url(
                "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.NetApp/netAppAccounts/{accountName}/capacityPools/{poolName}/volumes/{volumeName}/ransomwareReports/{ransomwareReportName}/clearSuspects",
                **self.url_parameters
            )

        @property
        def method(self):
            return "POST"

        @property
        def error_format(self):
            return "MgmtErrorFormat"

        @property
        def url_parameters(self):
            parameters = {
                **self.serialize_url_param(
                    "accountName", self.ctx.args.account_name,
                    required=True,
                ),
                **self.serialize_url_param(
                    "poolName", self.ctx.args.pool_name,
                    required=True,
                ),
                **self.serialize_url_param(
                    "ransomwareReportName", self.ctx.args.ransomware_report_name,
                    required=True,
                ),
                **self.serialize_url_param(
                    "resourceGroupName", self.ctx.args.resource_group,
                    required=True,
                ),
                **self.serialize_url_param(
                    "subscriptionId", self.ctx.subscription_id,
                    required=True,
                ),
                **self.serialize_url_param(
                    "volumeName", self.ctx.args.volume_name,
                    required=True,
                ),
            }
            return parameters

        @property
        def query_parameters(self):
            parameters = {
                **self.serialize_query_param(
                    "api-version", "2026-04-01",
                    required=True,
                ),
            }
            return parameters

        @property
        def header_parameters(self):
            parameters = {
                **self.serialize_header_param(
                    "Content-Type", "application/json",
                ),
            }
            return parameters

        @property
        def content(self):
            _content_value, _builder = self.new_content_builder(
                self.ctx.args,
                typ=AAZObjectType,
                typ_kwargs={"flags": {"required": True, "client_flatten": True}}
            )
            _builder.set_prop("extensions", AAZListType, ".extensions", typ_kwargs={"flags": {"required": True}})
            _builder.set_prop("resolution", AAZStrType, ".resolution", typ_kwargs={"flags": {"required": True}})

            extensions = _builder.get(".extensions")
            if extensions is not None:
                extensions.set_elements(AAZStrType, ".")

            return self.serialize_content(_content_value)


class _ClearSuspectHelper:
    """Helper class for ClearSuspect"""


__all__ = ["ClearSuspect"]



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import unittest

from azure.cli.testsdk import ScenarioTest, ResourceGroupPreparer, StorageAccountPreparer
from azure.cli.testsdk.decorators import serial_test


class CognitiveServicesByoxTests(ScenarioTest):
    @ResourceGroupPreparer()
    @StorageAccountPreparer(parameter_name='storage_account', allow_shared_key_access=True)    
    def test_cognitiveservices_user_owned_storage(self, resource_group, storage_account):
        
        # The subscription we are running in requires the byoxPreview feature registered.
        # How to request it: https://learn.microsoft.com/en-us/azure/ai-services/speech-service/bring-your-own-storage-speech-resource?tabs=azure-cli#request-access-to-byos-for-your-azure-subscriptions
        reg = self.cmd('az feature show --name byoxPreview --namespace  Microsoft.CognitiveServices',
                                    checks=[self.check('properties.state', 'Registered')])
            
        sname = self.create_random_name(prefix='cs_cli_test_', length=16)
        self.kwargs.update({
            'sname': sname,
            'kind': 'SpeechServices',
            'sku': 'S0',
            'location': 'WESTEUROPE',
            'stgname': storage_account,
        })

        stgacct = self.cmd('az storage account show -n {stgname} -g {rg}').get_output_in_json()
        self.kwargs.update( {
            'storageIds': '[{\\\"resourceId\\\":\\\"' + stgacct['id'] + '\\\"}]'
        })

        # test to create cognitive services account
        self.cmd('az cognitiveservices account create -n {sname} -g {rg} --kind {kind} --sku {sku} -l {location} '
                 '--assign-identity --storage {storageIds}  --yes',
                 checks=[self.check('name', '{sname}'),
                         self.check('location', '{location}'),
                         self.check('sku.name', '{sku}'),
                         self.check('properties.provisioningState', 'Succeeded')])

        account = self.cmd('az cognitiveservices account show -n {sname} -g {rg}').get_output_in_json()

        self.assertEqual(account['identity']['type'], 'SystemAssigned')
        self.assertEqual(len(account['properties']['userOwnedStorage']), 1)
        self.assertEqual(account['properties']['userOwnedStorage'][0]['resourceId'], stgacct['id'])

        # delete the cognitive services account
        ret = self.cmd('az cognitiveservices account delete -n {sname} -g {rg}')
        self.assertEqual(ret.exit_code, 0)


    @serial_test()
    @ResourceGroupPreparer()
    def test_cognitiveservices_encryption(self, resource_group):
        sname = self.create_random_name(prefix='cs_cli_test_', length=16)

        self.kwargs.update({
            'sname': sname,
            'kind': 'FormRecognizer',
            'sku': 'S0',
            'location': 'SOUTHCENTRALUS',
            'encryption': '{\\\"keySource\\\":\\\"Microsoft.CognitiveServices\\\"}'
        })

        # test to create cognitive services account
        self.cmd('az cognitiveservices account create -n {sname} -g {rg} --kind {kind} --sku {sku} -l {location} '
                 '--assign-identity --encryption {encryption}  --yes',
                 checks=[self.check('name', '{sname}'),
                         self.check('location', '{location}'),
                         self.check('sku.name', '{sku}')])


        account = self.cmd('az cognitiveservices account show -n {sname} -g {rg}').get_output_in_json()


        self.assertEqual(account['identity']['type'], 'SystemAssigned')
        self.assertTrue(account['properties']['encryption'] is not None)
        self.assertEqual(account['properties']['encryption']['keySource'], 'Microsoft.CognitiveServices')

        # delete the cognitive services account
        ret = self.cmd('az cognitiveservices account delete -n {sname} -g {rg}')
        self.assertEqual(ret.exit_code, 0)

        self.kwargs.update({
            'sname': self.create_random_name(prefix='cs_cli_test_', length=16)
        })

        # test to create cognitive services account
        self.cmd('az cognitiveservices account create -n {sname} -g {rg} --kind {kind} --sku {sku} -l {location} '
                 '--assign-identity --yes',
                 checks=[self.check('name', '{sname}'),
                         self.check('location', '{location}'),
                         self.check('sku.name', '{sku}'),
                         self.check('properties.provisioningState', 'Succeeded')])

        self.cmd('az cognitiveservices account update -n {sname} -g {rg} --encryption {encryption}')

        account = self.cmd('az cognitiveservices account show -n {sname} -g {rg}').get_output_in_json()
        self.assertEqual(account['identity']['type'], 'SystemAssigned')
        self.assertTrue(account['properties']['encryption'] is not None)
        self.assertEqual(account['properties']['encryption']['keySource'], 'Microsoft.CognitiveServices')

        # delete the cognitive services account
        ret = self.cmd('az cognitiveservices account delete -n {sname} -g {rg}')
        self.assertEqual(ret.exit_code, 0)


if __name__ == '__main__':
    unittest.main()



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import os

from azure.mgmt.core.tools import parse_resource_id

from azure.cli.core.azclierror import ValidationError
from azure.cli.testsdk.scenario_tests import AllowLargeResponse
from azure.cli.testsdk import (ScenarioTest, ResourceGroupPreparer, JMESPathCheck, LogAnalyticsWorkspacePreparer)

from azure.cli.command_modules.containerapp.tests.latest.common import TEST_LOCATION, write_test_file, \
    clean_up_test_file
from .utils import create_containerapp_env, prepare_containerapp_env_for_app_e2e_tests

TEST_DIR = os.path.abspath(os.path.join(os.path.abspath(__file__), '..'))
# flake8: noqa
# noqa
# pylint: skip-file


class ContainerAppJobsCRUDOperationsTest(ScenarioTest):
    def __init__(self, *arg, **kwargs):
        super().__init__(*arg, random_config_dir=True, **kwargs)

    @AllowLargeResponse(8192)
    @ResourceGroupPreparer(location="northcentralus")
    # test for CRUD operations on Container App Job resource with trigger type as manual
    def test_containerapp_manualjob_crudoperations_e2e(self, resource_group):
        import requests
        
        self.cmd('configure --defaults location={}'.format(TEST_LOCATION))

        env_id = prepare_containerapp_env_for_app_e2e_tests(self)
        env_rg = parse_resource_id(env_id).get('resource_group')
        env_name = parse_resource_id(env_id).get('name')
        job = self.create_random_name(prefix='job1', length=24)

        ## test for CRUD operations on Container App Job resource with trigger type as manual
        # create a Container App Job resource with trigger type as manual
        self.cmd("az containerapp job create --resource-group {} --name {} --environment {} --replica-timeout 200 --replica-retry-limit 2 --trigger-type manual --parallelism 1 --replica-completion-count 1 --image mcr.microsoft.com/k8se/quickstart-jobs:latest --cpu '0.25' --memory '0.5Gi'".format(resource_group, job, env_id))

        # verify the container app job resource
        self.cmd("az containerapp job show --resource-group {} --name {}".format(resource_group, job), checks=[
            JMESPathCheck('name', job),
            JMESPathCheck('properties.configuration.replicaTimeout', 200),
            JMESPathCheck('properties.configuration.replicaRetryLimit', 2),
            JMESPathCheck('properties.configuration.triggerType', "manual", case_sensitive=False),
        ])

        # get list of Container App Jobs
        jobs_list = self.cmd("az containerapp job list --resource-group {}".format(resource_group)).get_output_in_json()
        self.assertTrue(len(jobs_list) == 1)

        # update the Container App Job resource
        self.cmd("az containerapp job update --resource-group {} --name {} --replica-timeout 300 --replica-retry-limit 1 --image mcr.microsoft.com/k8se/quickstart-jobs:latest --cpu '0.5' --memory '1.0Gi'".format(resource_group, job), checks=[
            JMESPathCheck('name', job),
            JMESPathCheck('properties.provisioningState', "Succeeded"),
            JMESPathCheck('properties.configuration.replicaTimeout', 300),
            JMESPathCheck('properties.configuration.replicaRetryLimit', 1),
            JMESPathCheck('properties.configuration.triggerType', "manual", case_sensitive=False),
            JMESPathCheck('properties.template.containers[0].image', "mcr.microsoft.com/k8se/quickstart-jobs:latest"),
            JMESPathCheck('properties.template.containers[0].resources.cpu', "0.5"),
            JMESPathCheck('properties.template.containers[0].resources.memory', "1Gi"),
        ])

        # verify the updated Container App Job resource
        self.cmd("az containerapp job show --resource-group {} --name {}".format(resource_group, job), checks=[
            JMESPathCheck('name', job),
            JMESPathCheck('properties.configuration.replicaTimeout', 300),
            JMESPathCheck('properties.configuration.replicaRetryLimit', 1),
            JMESPathCheck('properties.configuration.triggerType', "manual", case_sensitive=False),
            JMESPathCheck('properties.template.containers[0].image', "mcr.microsoft.com/k8se/quickstart-jobs:latest"),
            JMESPathCheck('properties.template.containers[0].resources.cpu', "0.5"),
            JMESPathCheck('properties.template.containers[0].resources.memory', "1Gi"),
        ])

        # delete the Container App Job resource
        self.cmd("az containerapp job delete --resource-group {} --name {} --yes".format(resource_group, job))

        # verify the Container App Job resource is deleted
        jobs_list = self.cmd("az containerapp job list --resource-group {}".format(resource_group)).get_output_in_json()
        self.assertTrue(len(jobs_list) == 0)
        
        ## test for CRUD operations on Container App Job resource with trigger type as schedule
        job2 = self.create_random_name(prefix='job2', length=24)

        # create a Container App Job resource with trigger type as schedule
        self.cmd("az containerapp job create --resource-group {} --name {} --environment {} --replica-timeout 200 --replica-retry-limit 2 --trigger-type schedule --parallelism 1 --replica-completion-count 1 --cron-expression '*/5 * * * *' --image mcr.microsoft.com/k8se/quickstart-jobs:latest --cpu '0.25' --memory '0.5Gi'".format(resource_group, job2, env_id))

        # verify the container app job resource
        self.cmd("az containerapp job show --resource-group {} --name {}".format(resource_group, job2), checks=[
            JMESPathCheck('name', job2),
            JMESPathCheck('properties.configuration.replicaTimeout', 200),
            JMESPathCheck('properties.configuration.replicaRetryLimit', 2),
            JMESPathCheck('properties.configuration.triggerType', "schedule", case_sensitive=False),
            JMESPathCheck('properties.configuration.scheduleTriggerConfig.cronExpression', "*/5 * * * *"),
        ])

        # get list of Container App Jobs
        jobs_list = self.cmd("az containerapp job list --resource-group {}".format(resource_group)).get_output_in_json()
        self.assertTrue(len(jobs_list) == 1)

        # update the Container App Job resource
        self.cmd("az containerapp job update --resource-group {} --name {} --replica-timeout 300 --replica-retry-limit 1 --cron-expression '*/10 * * * *' --image mcr.microsoft.com/k8se/quickstart-jobs:latest --cpu '0.5' --memory '1.0Gi'".format(resource_group, job2), checks=[
            JMESPathCheck('properties.provisioningState', "Succeeded"),
            JMESPathCheck('properties.configuration.replicaTimeout', 300),
            JMESPathCheck('properties.configuration.replicaRetryLimit', 1),
            JMESPathCheck('properties.configuration.triggerType', "schedule", case_sensitive=False),
            JMESPathCheck('properties.configuration.scheduleTriggerConfig.cronExpression', "*/10 * * * *"),
            JMESPathCheck('properties.template.containers[0].image', "mcr.microsoft.com/k8se/quickstart-jobs:latest"),
            JMESPathCheck('properties.template.containers[0].resources.cpu', "0.5"),
            JMESPathCheck('properties.template.containers[0].resources.memory', "1Gi"),
        ])

        # verify the updated Container App Job resource
        self.cmd("az containerapp job show --resource-group {} --name {}".format(resource_group, job2), checks=[
            JMESPathCheck('name', job2),
            JMESPathCheck('properties.configuration.replicaTimeout', 300),
            JMESPathCheck('properties.configuration.replicaRetryLimit', 1),
            JMESPathCheck('properties.configuration.triggerType', "schedule", case_sensitive=False),
            JMESPathCheck('properties.configuration.scheduleTriggerConfig.cronExpression', "*/10 * * * *"),
            JMESPathCheck('properties.template.containers[0].image', "mcr.microsoft.com/k8se/quickstart-jobs:latest"),
            JMESPathCheck('properties.template.containers[0].resources.cpu', "0.5"),
            JMESPathCheck('properties.template.containers[0].resources.memory', "1Gi"),
        ])

    @AllowLargeResponse(8192)
    @ResourceGroupPreparer(location="northcentralus")
    # test for CRUD operations on Container App Job resource with trigger type as manual
    def test_containerappjob_list(self, resource_group):
        self.cmd('configure --defaults location={}'.format(TEST_LOCATION))

        env_id = prepare_containerapp_env_for_app_e2e_tests(self)

        for i in range(30):
            job = self.create_random_name(prefix=f'job{i}', length=24)
            self.cmd(
                "az containerapp job create --resource-group {} --name {} --environment {} --replica-timeout 200 --replica-retry-limit 2 --trigger-type manual --parallelism 1 --replica-completion-count 1 --image mcr.microsoft.com/k8se/quickstart-jobs:latest --cpu '0.25' --memory '0.5Gi' --no-wait".format(
                    resource_group, job, env_id))

        # get list of Container App Jobs
        jobs_list = self.cmd("az containerapp job list --resource-group {}".format(resource_group)).get_output_in_json()
        self.assertTrue(len(jobs_list) == 30)

        for job_item in jobs_list:
            # delete the Container App Job resource
            self.cmd("az containerapp job delete --resource-group {} --name {} --yes --no-wait".format(resource_group, job_item['name']))

    @AllowLargeResponse(8192)
    @ResourceGroupPreparer(location="westeurope")
    def test_containerapp_manualjob_private_registry_port(self, resource_group):
        self.cmd('configure --defaults location={}'.format(TEST_LOCATION))
        acr = self.create_random_name(prefix='acr', length=24)
        image_source = "mcr.microsoft.com/k8se/quickstart:latest"

        env_id = prepare_containerapp_env_for_app_e2e_tests(self)

        self.cmd(f'acr create --sku basic -n {acr} -g {resource_group} --admin-enabled')
        self.cmd(f'acr import -n {acr} --source {image_source}')
        password = self.cmd(f'acr credential show -n {acr} --query passwords[0].value').get_output_in_json()

        app = self.create_random_name(prefix='aca1', length=24)
        image_name = f"{acr}.azurecr.io:443/k8se/quickstart:latest"

        self.cmd(
            f"containerapp job create -g {resource_group} -n {app} --image {image_name} --environment {env_id} --registry-server {acr}.azurecr.io:443 --registry-username {acr} --registry-password {password} --replica-timeout 200 --replica-retry-limit 2 --trigger-type schedule --parallelism 1 --replica-completion-count 1 --cron-expression '*/10 * * * *' ")

        self.cmd(f'containerapp job show -g {resource_group} -n {app}', checks=[
            JMESPathCheck("properties.provisioningState", "Succeeded"),
            JMESPathCheck("properties.configuration.registries[0].server", f"{acr}.azurecr.io:443"),
            JMESPathCheck("properties.template.containers[0].image", image_name),
            JMESPathCheck("properties.configuration.secrets[0].name", f"{acr}azurecrio-443-{acr}")
        ])

    @AllowLargeResponse(8192)
    @ResourceGroupPreparer(location="westeurope")
    def test_containerapp_manualjob_registry_acr_look_up_credentical(self, resource_group):
        self.cmd('configure --defaults location={}'.format(TEST_LOCATION))
        app = self.create_random_name(prefix='aca', length=24)
        acr = self.create_random_name(prefix='acr', length=24)
        image_source = "mcr.microsoft.com/k8se/quickstart:latest"

        env_id = prepare_containerapp_env_for_app_e2e_tests(self)

        self.cmd(f'acr create --sku basic -n {acr} -g {resource_group} --admin-enabled')
        self.cmd(f'acr import -n {acr} --source {image_source}')

        app = self.create_random_name(prefix='aca1', length=24)
        image_name = f"{acr}.azurecr.io/k8se/quickstart:latest"

        self.cmd(
            f"containerapp job create -g {resource_group} -n {app} --image {image_name} --environment {env_id} --registry-server {acr}.azurecr.io --replica-timeout 200 --replica-retry-limit 2 --trigger-type schedule --parallelism 1 --replica-completion-count 1 --cron-expression '*/10 * * * *' ")

        self.cmd(f'containerapp job show -g {resource_group} -n {app}', checks=[
            JMESPathCheck("properties.provisioningState", "Succeeded"),
            JMESPathCheck("properties.configuration.registries[0].server", f"{acr}.azurecr.io"),
            JMESPathCheck("properties.template.containers[0].image", image_name),
            JMESPathCheck("properties.configuration.secrets[0].name", f"{acr}azurecrio-{acr}")
        ])

    @AllowLargeResponse(8192)
    @ResourceGroupPreparer(location="northcentralus")
    def test_containerappjob_create_with_default_parallelism_and_completion_count(self, resource_group):
        """
        Test that creating a manual trigger job without specifying --parallelism and 
        --replica-completion-count works correctly and defaults to 1 for both values.
        Previously these would default to 0 which caused job creation to fail.
        """
        self.cmd('configure --defaults location={}'.format(TEST_LOCATION))

        env_id = prepare_containerapp_env_for_app_e2e_tests(self)
        job = self.create_random_name(prefix='job-defaults', length=24)

        # Create a manual trigger job WITHOUT --parallelism and --replica-completion-count
        # These should default to 1
        self.cmd("az containerapp job create --resource-group {} --name {} --environment {} --replica-timeout 200 --replica-retry-limit 2 --trigger-type manual --image mcr.microsoft.com/k8se/quickstart-jobs:latest --cpu '0.25' --memory '0.5Gi'".format(resource_group, job, env_id))

        # Verify the job was created successfully with default values of 1
        self.cmd("az containerapp job show --resource-group {} --name {}".format(resource_group, job), checks=[
            JMESPathCheck('name', job),
            JMESPathCheck('properties.provisioningState', "Succeeded"),
            JMESPathCheck('properties.configuration.triggerType', "manual", case_sensitive=False),
            JMESPathCheck('properties.configuration.manualTriggerConfig.parallelism', 1),
            JMESPathCheck('properties.configuration.manualTriggerConfig.replicaCompletionCount', 1),
        ])

        # Cleanup
        self.cmd("az containerapp job delete --resource-group {} --name {} --yes".format(resource_group, job))

    @AllowLargeResponse(8192)
    @ResourceGroupPreparer(location="northcentralus")
    @LogAnalyticsWorkspacePreparer(location="eastus", get_shared_key=True)
    def test_containerappjob_create_with_environment_id(self, resource_group, laworkspace_customer_id, laworkspace_shared_key):
        self.cmd('configure --defaults location={}'.format(TEST_LOCATION))

        env1 = self.create_random_name(prefix='env1', length=24)
        env2 = self.create_random_name(prefix='env2', length=24)
        job1 = self.create_random_name(prefix='yaml1', length=24)

        create_containerapp_env(self, env1, resource_group, logs_workspace=laworkspace_customer_id, logs_workspace_shared_key=laworkspace_shared_key)
        containerapp_env1 = self.cmd(
            'containerapp env show -g {} -n {}'.format(resource_group, env1)).get_output_in_json()

        create_containerapp_env(self, env2, resource_group, logs_workspace=laworkspace_customer_id, logs_workspace_shared_key=laworkspace_shared_key)
        containerapp_env2 = self.cmd(
            'containerapp env show -g {} -n {}'.format(resource_group, env2)).get_output_in_json()

        # the value in --yaml is used, warning for different value in --environmentId
        containerappjob_yaml_text = f"""
                    location: {TEST_LOCATION}
                    properties:
                        environmentId: {containerapp_env1["id"]}
                        configuration:
                            dapr: null
                            eventTriggerConfig: null
                            manualTriggerConfig:
                                parallelism: 1
                                replicaCompletionCount: 1
                            registries: null
                            replicaRetryLimit: 1
                            replicaTimeout: 100
                            scheduleTriggerConfig: null
                            secrets: null
                            triggerType: Manual
                        template:
                            containers:
                            - env:
                                - name: MY_ENV_VAR
                                  value: hello
                              image: mcr.microsoft.com/k8se/quickstart-jobs:latest
                              name: anfranci-azclitest-acaj1
                              resources:
                                cpu: 0.5
                                ephemeralStorage: 1Gi
                                memory: 1Gi
                            initContainers:
                            - command:
                                - /bin/sh
                                - -c
                                - sleep 150
                              image: k8seteste2e.azurecr.io/e2e-apps/kuar:green
                              name: simple-sleep-container
                              probes:
                              - type: liveness
                                httpGet:
                                    path: "/health"
                                    port: 8080
                                    httpHeaders:
                                        - name: "Custom-Header"
                                          value: "liveness probe"
                                initialDelaySeconds: 7
                                periodSeconds: 3
                                resources:
                                    cpu: "0.25"
                                    memory: 0.5Gi
                        workloadProfileName: null
                    """
        containerappjob_file_name = f"{self._testMethodName}_containerappjob.yml"

        write_test_file(containerappjob_file_name, containerappjob_yaml_text)
        self.cmd(
            f'containerapp job create -n {job1} -g {resource_group} --environment {env2} --yaml {containerappjob_file_name}',
            checks=[
                JMESPathCheck("properties.provisioningState", "Succeeded"),
                JMESPathCheck("properties.environmentId", containerapp_env1["id"]),
                JMESPathCheck("properties.configuration.triggerType", "Manual", case_sensitive=False),
                JMESPathCheck('properties.configuration.replicaTimeout', 100),
                JMESPathCheck('properties.configuration.replicaRetryLimit', 1),
                JMESPathCheck('properties.template.containers[0].image',
                              "mcr.microsoft.com/k8se/quickstart-jobs:latest"),
                JMESPathCheck('properties.template.containers[0].resources.cpu', "0.5"),
                JMESPathCheck('properties.template.containers[0].resources.memory', "1Gi"),
            ])

        self.cmd(f'containerapp job show -g {resource_group} -n {job1}', checks=[
            JMESPathCheck("properties.provisioningState", "Succeeded"),
            JMESPathCheck("properties.environmentId", containerapp_env1["id"]),
            JMESPathCheck("properties.configuration.triggerType", "Manual", case_sensitive=False),
            JMESPathCheck('properties.configuration.replicaTimeout', 100),
            JMESPathCheck('properties.configuration.replicaRetryLimit', 1),
            JMESPathCheck('properties.template.containers[0].image', "mcr.microsoft.com/k8se/quickstart-jobs:latest"),
            JMESPathCheck('properties.template.containers[0].resources.cpu', "0.5"),
            JMESPathCheck('properties.template.containers[0].resources.memory', "1Gi"),
        ])

        # test container app job update with yaml
        containerappjob_yaml_text = f"""
                    location: {TEST_LOCATION}
                    properties:
                        configuration:
                            dapr: null
                            eventTriggerConfig: null
                            manualTriggerConfig:
                                parallelism: 1
                                replicaCompletionCount: 1
                            registries: null
                            replicaRetryLimit: 1
                            replicaTimeout: 200
                            scheduleTriggerConfig: null
                            secrets: null
                            triggerType: Manual
                        template:
                            containers:
                            - env:
                                - name: MY_ENV_VAR
                                  value: hello
                              image: mcr.microsoft.com/k8se/quickstart-jobs:latest
                              name: anfranci-azclitest-acaj1
                              resources:
                                cpu: 0.75
                                ephemeralStorage: 1Gi
                                memory: 1.5Gi
                            initContainers:
                            - command:
                                - /bin/sh
                                - -c
                                - sleep 150
                              image: k8seteste2e.azurecr.io/e2e-apps/kuar:green
                              name: simple-sleep-container
                            probes:
                            - type: liveness
                              httpGet:
                                path: "/health"
                                port: 8080
                                httpHeaders:
                                    - name: "Custom-Header"
                                      value: "liveness probe"
                                initialDelaySeconds: 7
                                periodSeconds: 3
                                resources:
                                    cpu: "0.25"
                                    memory: 0.5Gi
                    """
        write_test_file(containerappjob_file_name, containerappjob_yaml_text)
        job2 = self.create_random_name(prefix='yaml2', length=24)
        self.cmd(
            f'containerapp job create -n {job2} -g {resource_group} --environment {env2} --yaml {containerappjob_file_name}',
            checks=[
                JMESPathCheck("properties.provisioningState", "Succeeded"),
                JMESPathCheck("properties.environmentId", containerapp_env2["id"]),
                JMESPathCheck("properties.configuration.triggerType", "Manual", case_sensitive=False),
                JMESPathCheck('properties.configuration.replicaTimeout', 200),
                JMESPathCheck('properties.configuration.replicaRetryLimit', 1),
                JMESPathCheck('properties.template.containers[0].image',
                              "mcr.microsoft.com/k8se/quickstart-jobs:latest"),
                JMESPathCheck('properties.template.containers[0].resources.cpu', "0.75"),
                JMESPathCheck('properties.template.containers[0].resources.memory', "1.5Gi"),
            ])

        self.cmd(f'containerapp job list -g {resource_group}', checks=[
            JMESPathCheck("length(@)", 2),
        ])
        clean_up_test_file(containerappjob_file_name)

    @AllowLargeResponse(8192)
    @ResourceGroupPreparer(location="northcentralus")
    def test_containerappjob_update_min_max_executions_zero_e2e(self, resource_group):
        """Test that min_executions and max_executions can be set to 0 in event-triggered jobs"""
        self.cmd('configure --defaults location={}'.format(TEST_LOCATION))

        env_id = prepare_containerapp_env_for_app_e2e_tests(self)
        job = self.create_random_name(prefix='eventjob', length=24)

        # Create a Container App Job resource with trigger type as event
        self.cmd("az containerapp job create --resource-group {} --name {} --environment {} --replica-timeout 200 --replica-retry-limit 2 --trigger-type event --parallelism 1 --replica-completion-count 1 --min-executions 1 --max-executions 5 --polling-interval 30 --scale-rule-name azure-servicebus --scale-rule-type azure-servicebus --scale-rule-metadata topicName=my-topic messageCount=5 --scale-rule-auth secretRef=connection-string-secret --image mcr.microsoft.com/k8se/quickstart-jobs:latest --cpu '0.25' --memory '0.5Gi'".format(resource_group, job, env_id))

        # Verify the initial container app job resource
        self.cmd("az containerapp job show --resource-group {} --name {}".format(resource_group, job), checks=[
            JMESPathCheck('name', job),
            JMESPathCheck('properties.configuration.replicaTimeout', 200),
            JMESPathCheck('properties.configuration.replicaRetryLimit', 2),
            JMESPathCheck('properties.configuration.triggerType', "event", case_sensitive=False),
            JMESPathCheck('properties.configuration.eventTriggerConfig.scale.minExecutions', 1),
            JMESPathCheck('properties.configuration.eventTriggerConfig.scale.maxExecutions', 5),
            JMESPathCheck('properties.configuration.eventTriggerConfig.scale.pollingInterval', 30),
        ])

        # Test 1: Update min_executions to 0
        self.cmd("az containerapp job update --resource-group {} --name {} --min-executions 0".format(resource_group, job), checks=[
            JMESPathCheck('name', job),
            JMESPathCheck('properties.provisioningState', "Succeeded"),
            JMESPathCheck('properties.configuration.eventTriggerConfig.scale.minExecutions', 0),
            JMESPathCheck('properties.configuration.eventTriggerConfig.scale.maxExecutions', 5),  # Should remain unchanged
        ])

        # Verify min_executions is set to 0
        self.cmd("az containerapp job show --resource-group {} --name {}".format(resource_group, job), checks=[
            JMESPathCheck('properties.configuration.eventTriggerConfig.scale.minExecutions', 0),
            JMESPathCheck('properties.configuration.eventTriggerConfig.scale.maxExecutions', 5),
        ])

        # Test 2: Update max_executions to 0
        self.cmd("az containerapp job update --resource-group {} --name {} --max-executions 0".format(resource_group, job), checks=[
            JMESPathCheck('name', job),
            JMESPathCheck('properties.provisioningState', "Succeeded"),
            JMESPathCheck('properties.configuration.eventTriggerConfig.scale.minExecutions', 0),  # Should remain unchanged
            JMESPathCheck('properties.configuration.eventTriggerConfig.scale.maxExecutions', 0),
        ])

        # Verify max_executions is set to 0
        self.cmd("az containerapp job show --resource-group {} --name {}".format(resource_group, job), checks=[
            JMESPathCheck('properties.configuration.eventTriggerConfig.scale.minExecutions', 0),
            JMESPathCheck('properties.configuration.eventTriggerConfig.scale.maxExecutions', 0),
        ])

        # Test 3: Update both min_executions and max_executions to 0 in a single command
        # First reset to non-zero values
        self.cmd("az containerapp job update --resource-group {} --name {} --min-executions 2 --max-executions 10".format(resource_group, job), checks=[
            JMESPathCheck('properties.configuration.eventTriggerConfig.scale.minExecutions', 2),
            JMESPathCheck('properties.configuration.eventTriggerConfig.scale.maxExecutions', 10),
        ])

        # Now set both to 0
        self.cmd("az containerapp job update --resource-group {} --name {} --min-executions 0 --max-executions 0".format(resource_group, job), checks=[
            JMESPathCheck('name', job),
            JMESPathCheck('properties.provisioningState', "Succeeded"),
            JMESPathCheck('properties.configuration.eventTriggerConfig.scale.minExecutions', 0),
            JMESPathCheck('properties.configuration.eventTriggerConfig.scale.maxExecutions', 0),
        ])

        # Final verification that both are 0
        self.cmd("az containerapp job show --resource-group {} --name {}".format(resource_group, job), checks=[
            JMESPathCheck('properties.configuration.eventTriggerConfig.scale.minExecutions', 0),
            JMESPathCheck('properties.configuration.eventTriggerConfig.scale.maxExecutions', 0),
            JMESPathCheck('properties.configuration.triggerType', "event", case_sensitive=False),
        ])

        # Test 4: Update back to positive values to ensure the fix doesn't break normal functionality
        self.cmd("az containerapp job update --resource-group {} --name {} --min-executions 3 --max-executions 8".format(resource_group, job), checks=[
            JMESPathCheck('properties.configuration.eventTriggerConfig.scale.minExecutions', 3),
            JMESPathCheck('properties.configuration.eventTriggerConfig.scale.maxExecutions', 8),
        ])

        # Cleanup
        self.cmd("az containerapp job delete --resource-group {} --name {} --yes".format(resource_group, job))

        # Verify the Container App Job resource is deleted
        jobs_list = self.cmd("az containerapp job list --resource-group {}".format(resource_group)).get_output_in_json()
        job_names = [job_item['name'] for job_item in jobs_list]
        self.assertNotIn(job, job_names)
            



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from azure.cli.testsdk import ScenarioTest, ResourceGroupPreparer


class NatGatewayScenarioTests(ScenarioTest):

    @ResourceGroupPreparer(location='eastus2')
    def test_natgateway_basic(self, resource_group, resource_group_location):

        self.kwargs.update({
            'name': "ng1",
            'name2': "ng2",
            'idle_timeout': 4,
            'sku': "Standard",
            'ip_addr': "pip",
            'ip_prefix': "prefix",
            'idle_timeout_updated': 5,
            'zone': 2,
            'location': resource_group_location,
            'resource_type': 'Microsoft.Network/NatGateways',
            'tags': 'foo=bar'
        })

        # create public ip address
        self.cmd('az network public-ip create -g {rg} -n {ip_addr} --location {location} --zone {zone} --sku Standard')

        # create public ip prefix
        self.cmd('az network public-ip prefix create --length 29 --location {location} --name {ip_prefix} --resource-group {rg} --zone {zone}')

        self.cmd('az network nat gateway create --resource-group {rg} --public-ip-prefixes {ip_prefix} --name {name} --location {location} --public-ip-addresses {ip_addr} --idle-timeout {idle_timeout} --zone {zone} --tags {tags}', checks=[
            self.check('resourceGroup', '{rg}'),
            self.check('idleTimeoutInMinutes', '{idle_timeout}'),
            self.check("contains(publicIpAddresses[0].id, '{ip_addr}')", True),
            self.check("contains(publicIpPrefixes[0].id, '{ip_prefix}')", True),
            self.check('sku.name', 'Standard'),
            self.check('location', '{location}'),
            self.check('zones[0]', '{zone}'),
            self.check('tags', {'foo': 'bar'})
        ])
        self.cmd('az network nat gateway update -g {rg} --name {name} --idle-timeout {idle_timeout_updated}',
                 checks=self.check('idleTimeoutInMinutes', 5))
        self.cmd('az network nat gateway list -g {rg}',
                 checks=self.check('length(@)', 1))
        self.cmd('az network nat gateway show --resource-group {rg} --name {name}',
                 checks=self.check('name', '{name}'))

        # delete and verify item is removed
        self.cmd('az network nat gateway delete --resource-group {rg} --name {name}')
        self.cmd('az network nat gateway list -g {rg}',
                 checks=self.check('length(@)', 0))

        # test standardv2 sku
        self.cmd('az network nat gateway create -g {rg} -n {name2} --sku StandardV2', checks=[
            self.check('sku.name', 'StandardV2')
        ])
        self.cmd('az network nat gateway delete -g {rg} -n {name2}')

    @ResourceGroupPreparer(location='eastus2')
    def test_natgateway_pipv6_param_formats(self, resource_group, resource_group_location):
        """Test that --pip-addresses-v6 / --pip-prefixes-v6 accept names, resource IDs, and JSON objects."""
        self.kwargs.update({
            'name': 'ng-v6-formats',
            'location': resource_group_location,
            'ip_v4': 'pipv4',
            'ip_v6_1': 'pipv6-1',
            'ip_v6_2': 'pipv6-2',
            'prefix_v6_1': 'prefixv6-1',
            'prefix_v6_2': 'prefixv6-2',
        })

        self.cmd(
            'az network public-ip create -g {rg} -n {ip_v4} '
            '--location {location} --sku StandardV2 --tier Regional --allocation-method Static'
        )

        result = self.cmd(
            'az network public-ip create -g {rg} -n {ip_v6_1} '
            '--location {location} --sku StandardV2 --version IPv6 '
            '--tier Regional --allocation-method Static'
        ).get_output_in_json()
        self.kwargs['ip_v6_1_id'] = result['publicIp']['id']

        result = self.cmd(
            'az network public-ip create -g {rg} -n {ip_v6_2} '
            '--location {location} --sku StandardV2 --version IPv6 '
            '--tier Regional --allocation-method Static'
        ).get_output_in_json()
        self.kwargs['ip_v6_2_id'] = result['publicIp']['id']

        result = self.cmd(
            'az network public-ip prefix create -g {rg} -n {prefix_v6_1} '
            '--location {location} --length 127 '
            '--sku StandardV2 --version IPv6 --tier Regional'
        ).get_output_in_json()
        self.kwargs['prefix_v6_1_id'] = result['id']

        result = self.cmd(
            'az network public-ip prefix create -g {rg} -n {prefix_v6_2} '
            '--location {location} --length 127 '
            '--sku StandardV2 --version IPv6 --tier Regional'
        ).get_output_in_json()
        self.kwargs['prefix_v6_2_id'] = result['id']

        self.kwargs['ip_v6_json'] = '[{"id": "' + self.kwargs['ip_v6_1_id'] + '"}]'

        # create with single resource name
        self.cmd(
            'az network nat gateway create -g {rg} -n {name} --sku StandardV2 '
            '--public-ip-addresses {ip_v4} '
            '--pip-addresses-v6 {ip_v6_1} '
            '--pip-prefixes-v6 {prefix_v6_1}',
            checks=[
                self.check('sku.name', 'StandardV2'),
                self.check("length(publicIpAddressesV6)", 1),
                self.check("contains(publicIpAddressesV6[0].id, '{ip_v6_1}')", True),
                self.check("contains(publicIpPrefixesV6[0].id, '{prefix_v6_1}')", True),
            ]
        )

        # update: multiple names
        self.cmd(
            'az network nat gateway update -g {rg} -n {name} '
            '--pip-addresses-v6 {ip_v6_1} {ip_v6_2} '
            '--pip-prefixes-v6 {prefix_v6_1} {prefix_v6_2}',
            checks=[
                self.check("length(publicIpAddressesV6)", 2),
                self.check("contains(publicIpAddressesV6[0].id, '{ip_v6_1}')", True),
                self.check("contains(publicIpAddressesV6[1].id, '{ip_v6_2}')", True),
                self.check("length(publicIpPrefixesV6)", 2),
                self.check("contains(publicIpPrefixesV6[0].id, '{prefix_v6_1}')", True),
                self.check("contains(publicIpPrefixesV6[1].id, '{prefix_v6_2}')", True),
            ]
        )

        # multiple full resource IDs
        self.cmd(
            'az network nat gateway update -g {rg} -n {name} '
            '--pip-addresses-v6 {ip_v6_1_id} {ip_v6_2_id} '
            '--pip-prefixes-v6 {prefix_v6_1_id} {prefix_v6_2_id}',
            checks=[
                self.check("contains(publicIpAddressesV6[0].id, '{ip_v6_1}')", True),
                self.check("contains(publicIpAddressesV6[1].id, '{ip_v6_2}')", True),
                self.check("contains(publicIpPrefixesV6[0].id, '{prefix_v6_1}')", True),
                self.check("contains(publicIpPrefixesV6[1].id, '{prefix_v6_2}')", True),
            ]
        )

        # legacy JSON format
        self.cmd(
            "az network nat gateway update -g {rg} -n {name} "
            "--pip-addresses-v6 '{ip_v6_json}'",
            checks=[
                self.check("length(publicIpAddressesV6)", 1),
                self.check("contains(publicIpAddressesV6[0].id, '{ip_v6_1}')", True),
            ]
        )

        self.cmd('az network nat gateway delete -g {rg} -n {name}')

    @ResourceGroupPreparer(location='eastus2')
    def test_natgateway_empty_create(self, resource_group, resource_group_location):
        self.kwargs.update({
            'name': "ng1",
            'idle_timeout': 4,
            'sku': "Standard",
            'ip_addr': "pip",
            'ip_prefix': "prefix",
            'idle_timeout_updated': 5,
            'zone': 2,
            'location': resource_group_location,
            'resource_type': 'Microsoft.Network/NatGateways',
            'tags': 'foo=bar'
        })
        self.cmd(
            'az network nat gateway create --resource-group {rg} --name {name} --location {location} --idle-timeout {idle_timeout} --zone {zone} --tags {tags}',
            checks=[
                self.check('resourceGroup', '{rg}'),
                self.check('idleTimeoutInMinutes', '{idle_timeout}'),
                self.check('sku.name', 'Standard'),
                self.check('location', '{location}'),
                self.check('zones[0]', '{zone}'),
                self.check('tags', {'foo': 'bar'})
            ])
